@@ -1,7 +1,7 @@
 /*
  * Alternate global C function prototypes for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  */
 
 /* command function list */
@@ -13,14 +13,14 @@ int		save_cmd(), taylor_cmd(), limit_cmd(), echo_cmd(), plot_cmd();
 int		copy_cmd(), divide_cmd(), pause_cmd(), version_cmd();
 int		edit_cmd(), real_cmd(), imaginary_cmd(), tally_cmd();
 int		roots_cmd(), set_cmd(), variables_cmd(), code_cmd(), optimize_cmd(), push_cmd();
-int		sum_cmd(), product_cmd(), integrate_cmd(), nintegrate_cmd(), laplace_cmd();
+int		sum_cmd(), product_cmd(), for_cmd(), integrate_cmd(), nintegrate_cmd(), laplace_cmd();
 
 /* various functions that don't return int */
 char		*dirname_win();
 char		*skip_space(), *skip_param();
 char		*get_string();
 char		*parse_equation(), *parse_section(), *parse_var(), *parse_var2(), *parse_expr();
-char		*list_expression(), *list_equation();
+char		*list_expression(), *list_equation(), *flist_equation_string();
 double		gcd(), gcd_verified(), my_round(), multiply_out_unique();
 long		decstrtol(), max_memory_usage();
 
