@@ -33,10 +33,10 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  */
 void
 error(str)
-const char	*str;
+const char	*str;		/* constant string to display */
 {
-	error_str = str;	/* must be a constant string, temporary strings don't work */
-#if	!SILENT
+	error_str = str;	/* save reference to str, must be a constant string, temporary strings don't work */
+#if	!SILENT && !LIBRARY
 	set_color(2);		/* set color to red */
 	printf("%s\n", str);
 	default_color();	/* restore to default color */
@@ -45,17 +45,18 @@ const char	*str;
 
 /*
  * Standard function to report a warning to the user.
+ * A warning is less serious than an error.
  */
 void
 warning(str)
-const char	*str;
+const char	*str;		/* constant string to display */
 {
-	warning_str = str;	/* must be a constant string, temporary strings don't work */
-#if	!SILENT
+	warning_str = str;	/* save reference to str, must be a constant string, temporary strings don't work */
+#if	!SILENT && !LIBRARY
 	if (debug_level >= 0) {
-		set_color(1);	/* set color to yellow */
+		set_color(1);		/* set color to yellow */
 		printf("Warning: %s\n", str);
-		default_color();
+		default_color();	/* restore to default color */
 	}
 #endif
 }
@@ -66,7 +67,7 @@ const char	*str;
  * There is no return.
  */
 void
-error_huge()
+error_huge(void)
 {
 	longjmp(jmp_save, 14);
 }
@@ -80,8 +81,9 @@ void
 error_bug(str)
 const char	*str;	/* constant string to display */
 {
-	error(str);		/* Display the passed error message. */
-#if	SILENT
+/* Return and display the passed error message in str. */
+	error(str);	/* str must be a constant string, temporary strings don't work */
+#if	SILENT || LIBRARY
 	printf("%s\n", str);
 #endif
 	printf(_("Please report this bug to the maintainers,\n"));
@@ -98,7 +100,7 @@ const char	*str;	/* constant string to display */
  * There is no return if an error message is displayed.
  */
 void
-check_err()
+check_err(void)
 {
 	switch (errno) {
 	case EDOM:
@@ -124,7 +126,7 @@ check_err()
  * Return true if the global integers screen_columns and/or screen_rows were set.
  */
 int
-get_screen_size()
+get_screen_size(void)
 {
 	int	rv = false;
 
@@ -151,6 +153,38 @@ get_screen_size()
 	return rv;
 }
 
+/*
+ * Allocate the display lines in the vscreen[] array.
+ * Call this before using vscreen[].
+ *
+ * Return true with vscreen[] allocated to TEXT_ROWS*current_columns characters if successful.
+ */
+int
+malloc_vscreen(void)
+{
+	int	i;
+
+	if (current_columns == 0 || ((screen_columns > 0) ? (current_columns != screen_columns) : (current_columns != TEXT_COLUMNS))) {
+		if (screen_columns > 0) {
+			current_columns = screen_columns;
+		} else {
+			current_columns = TEXT_COLUMNS;
+		}
+		for (i = 0; i < TEXT_ROWS; i++) {
+			if (vscreen[i]) {
+				free(vscreen[i]);
+			}
+			vscreen[i] = malloc(current_columns);
+			if (vscreen[i] == NULL) {
+				error(_("Out of memory (can't malloc(3))."));
+				current_columns = 0;
+				return false;
+			}
+		}
+	}
+	return true;
+}
+
 /*
  * Allocate the needed global expression storage arrays.
  * Each is static and can hold n_tokens elements.
@@ -163,7 +197,7 @@ get_screen_size()
  * Returns true if successful, otherwise Mathomatic cannot be used.
  */
 int
-init_mem()
+init_mem(void)
 {
 	if (n_tokens <= 0)
 		return false;
@@ -191,7 +225,7 @@ init_mem()
  * Inclusion of this routine was requested by Tam Hanna for use with Symbian OS.
  */
 void
-free_mem()
+free_mem(void)
 {
 	int	i;
 
@@ -221,7 +255,7 @@ free_mem()
  * This is called on startup and by process() to reset the global flags to the default state.
  */
 void
-init_gvars()
+init_gvars(void)
 {
 	domain_check = false;
 	high_prec = false;
@@ -246,7 +280,7 @@ init_gvars()
  * Clean up when processing is unexpectedly interrupted or terminated.
  */
 void
-clean_up()
+clean_up(void)
 {
 	int	i;
 
@@ -272,7 +306,7 @@ clean_up()
  * so that the next sign variables returned by next_sign() will be unique.
  */
 void
-set_sign_array()
+set_sign_array(void)
 {
 	int	i, j;
 
@@ -323,7 +357,7 @@ long	*vp;
  * Similar to a restart.
  */
 void
-clear_all()
+clear_all(void)
 {
 	int	i;
 
@@ -401,7 +435,7 @@ int	en;	/* equation space number */
  * Returns empty equation space number ready for use or -1 on error.
  */
 int
-alloc_next_espace()
+alloc_next_espace(void)
 {
 	int	i, n;
 
@@ -434,7 +468,7 @@ alloc_next_espace()
  * Return the number of the next empty equation space, otherwise don't return.
  */
 int
-next_espace()
+next_espace(void)
 {
 	int	i;
 
@@ -598,6 +632,10 @@ int		n;		/* expression length */
 	int		min1;
 	token_type	*p1, *ep;
 
+#if	DEBUG
+	if (expression == NULL)
+		error_bug("NULL pointer passed to min_level().");
+#endif
 	switch (n) {
 	case 1:
 		return expression[0].level;
@@ -730,7 +768,7 @@ int	i;	/* equation space number */
  * Return true and display a message if current equation space is empty.
  */
 int
-current_not_defined()
+current_not_defined(void)
 {
 	int	i;
 
@@ -763,6 +801,10 @@ int	n;		/* maximum size of "string" in bytes */
 	char	*cp;
 #endif
 
+#if	DEBUG
+	if (string == NULL)
+		error_bug("NULL pointer passed to get_string().");
+#endif
 	if (quiet_mode) {
 		prompt_str[0] = '\0';	/* don't display a prompt */
 	}
@@ -823,7 +865,7 @@ int	n;		/* maximum size of "string" in bytes */
  * Return false if "n" or not interactive.
  */
 int
-get_yes_no()
+get_yes_no(void)
 {
 	char	*cp;
 	char	buf[MAX_CMD_LEN];
@@ -865,7 +907,7 @@ int	en;	/* equation space number the result is in */
 		factor_int_sub(en);
 	}
 	free_result_str();
-#if	0	/* Set this to 1 to have multi-line, fraction format output from most Mathomatic commands in library mode. */
+#if	ROBOT_COMMAND	/* Set this to 1 to have multi-line, fraction format output from most Mathomatic commands in library mode. */
 	if (display2d) {
 		result_str = flist_equation_string(en);
 		if (result_str == NULL)
@@ -889,7 +931,7 @@ int	en;	/* equation space number the result is in */
  * in the symbolic math library.
  */
 void
-free_result_str()
+free_result_str(void)
 {
 	if (result_str) {
 		free(result_str);
@@ -1082,7 +1124,7 @@ char	*cp;
 			cp++;
 		}
 		cp = skip_space(cp);
-		if (isdelimiter(*cp)) {
+		if (*cp && isdelimiter(*cp)) {
 			cp = skip_space(cp + 1);
 		}
 	}
@@ -1090,9 +1132,8 @@ char	*cp;
 }
 
 /*
- * Compare strings up to the end or the first space or parameter delimiter.
- * Must be an exact match.
- * Compare is alphabetic case insensitive.
+ * Compare strings up to the end or the first space or parameter delimiter,
+ * ignoring alphabetic case.
  *
  * Returns zero on exact match, otherwise non-zero if strings are different.
  */
@@ -1102,6 +1143,10 @@ char	*cp1, *cp2;
 {
 	char	*cp1a, *cp2a;
 
+#if	DEBUG
+	if (cp1 == NULL || cp2 == NULL)
+		error_bug("NULL pointer passed to strcmp_tospace().");
+#endif
 	for (cp1a = cp1; *cp1a && !isdelimiter(*cp1a); cp1a++)
 		;
 	for (cp2a = cp2; *cp2a && !isdelimiter(*cp2a); cp2a++)
