@@ -49,7 +49,7 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 #define	STANDARD_SCREEN_COLUMNS	80			/* default number of columns of characters on the screen */
 #define	STANDARD_SCREEN_ROWS	24			/* default number of lines on the screen */
 #define	TEXT_ROWS	STANDARD_SCREEN_ROWS		/* number of lines per page in the symbolic math library */
-#define	TEXT_COLUMNS	STANDARD_SCREEN_COLUMNS		/* number of columns per page in the symbolic math library */
+#define	TEXT_COLUMNS	STANDARD_SCREEN_COLUMNS		/* default number of columns per page in the symbolic math library */
 
 #define	TMP_FILE	"/tmp/mathomatic.XXXXXX"	/* temporary file template for mkstemp(3) */
 
