@@ -50,13 +50,16 @@ int
 plot_cmd(cp)
 char	*cp;	/* the command line argument */
 {
-#define	APPEND(str)	{ if (strlen(str) + cl1_len < sizeof(cl1)) { strcpy(&cl1[cl1_len], str); cl1_len += strlen(str); } }
+#define	APPEND(str)	{ if (strlen(str) + cl1_len < sizeof(cl1)) { strcpy(&cl1[cl1_len], str); cl1_len += strlen(str); } else warning(_("Expression too large to plot; omitted.")); }
 
-	int	i, j, i1;
-	int	first_time = true;
-	int	cl1_len = 0, len;
-	char	*cp1, cl[MAX_CMD_LEN], cl1[MAX_CMD_LEN], *exp_str;
-	long	v, vx;			/* Mathomatic variables */
+	int		i1, i2;
+	int		start, stop;
+	int		first_time = true;
+	int		cl1_len = 0, cl2_len = 0, len;
+	char		*cp1, cl[MAX_CMD_LEN], cl1[MAX_CMD_LEN], cl2[MAX_CMD_LEN], *exp_str;
+	long		v, vx;			/* Mathomatic variables */
+	token_type	*equation;
+	int		*np;
 
 	if (security_level > 0) {
 		error(_("Command disabled by security level."));
@@ -64,32 +67,39 @@ char	*cp;	/* the command line argument */
 	}
 	do {
 		cp1 = cp;
-		if (!get_range(&cp, &i, &j)) {
+		if (!get_range(&cp, &start, &stop)) {
 			break;
 		}
 		if (cp != cp1 || first_time) {
-			for (i1 = i; i1 <= j; i1++) {
+			for (i1 = start; i1 <= stop; i1++) {
 				if (n_lhs[i1]) {
 					v = 0;
 					if (n_rhs[i1]) {
-						if (!no_vars(rhs[i1], n_rhs[i1], &v) && v) {
-							if (strcmp(var_name(v), "x") && strcmp(var_name(v), "t") && parse_var(&vx, "x")) {
-								list_var(v, 0);
-								fprintf(gfp, _("Renaming variable %s to x for gnuplot.\n"), var_str);
-								rename_var_in_es(i1, v, vx);
-							}
-						}
-						exp_str = list_expression(rhs[i1], n_rhs[i1], 3);
+						equation = rhs[i1];
+						np = &n_rhs[i1];
 					} else {
-						if (!no_vars(lhs[i1], n_lhs[i1], &v) && v) {
-							if (strcmp(var_name(v), "x") && strcmp(var_name(v), "t") && parse_var(&vx, "x")) {
-								list_var(v, 0);
-								fprintf(gfp, _("Renaming variable %s to x for gnuplot.\n"), var_str);
-								rename_var_in_es(i1, v, vx);
-							}
-						}
-						exp_str = list_expression(lhs[i1], n_lhs[i1], 3);
+						equation = lhs[i1];
+						np = &n_lhs[i1];
 					}
+					if (!no_vars(equation, *np, &v) && v) {
+						if (strcmp(var_name(v), "x") && strcmp(var_name(v), "t") && parse_var(&vx, "x")) {
+							list_var(v, 0);
+							fprintf(gfp, _("#%d: Renaming variable \"%s\" to x for gnuplot.\n"), i1 + 1, var_str);
+							rename_var_in_es(i1, v, vx);
+						}
+					}
+					if (n_rhs[i1] && !solved_equation(i1)) {
+						warning(_("Not a normally solved equation, plotting the RHS only."));
+					}
+					for (i2 = 0; i2 < *np; i2 += 2) {
+						if (equation[i2].kind == VARIABLE && (equation[i2].token.variable & VAR_MASK) == SIGN) {
+							warning(_("Plot expression contains sign variables; try \"simplify sign\" before plotting."));
+							break;
+						}
+					}
+					exp_str = list_expression(equation, *np, 3);
+					if (exp_str == NULL)
+						return false;
 					if (cl1_len) {
 						APPEND(", ");
 					}
@@ -105,12 +115,31 @@ char	*cp;	/* the command line argument */
 		error(_("No plot expression specified."));
 		return false;
 	}
-	if (strchr(cp, 'y') || strchr(cl1, 'y')) {
+	for (cl2_len = 0, i2 = 0; cp[i2]; i2++) {
+		if ((cl2_len + 2) >= sizeof(cl2)) {
+			error(_("Command line too long."));
+			return false;
+		}
+		switch (cp[i2]) {
+		case '^':
+			cl2[cl2_len] = '*';
+			cl2_len++;
+			cl2[cl2_len] = '*';
+			cl2_len++;
+			break;
+		default:
+			cl2[cl2_len] = cp[i2];
+			cl2_len++;
+			break;
+		}
+	}
+	cl2[cl2_len] = '\0';
+	if (strchr(cl2, 'y') || strchr(cl1, 'y')) {
 		fprintf(gfp, _("Performing 3D surface plot...\n"));
-		len = snprintf(cl, sizeof(cl), "echo '%s; splot %s %s'|gnuplot -persist", plot_prefix, cp, cl1);
+		len = snprintf(cl, sizeof(cl), "echo '%s; splot %s %s'|gnuplot -persist", plot_prefix, cl2, cl1);
 	} else {
 		fprintf(gfp, _("Performing 2D plot...\n"));
-		len = snprintf(cl, sizeof(cl), "echo '%s; plot %s %s'|gnuplot -persist", plot_prefix, cp, cl1);
+		len = snprintf(cl, sizeof(cl), "echo '%s; plot %s %s'|gnuplot -persist", plot_prefix, cl2, cl1);
 	}
 	if (len >= sizeof(cl)) {
 		error(_("gnuplot command line too long."));
@@ -162,7 +191,7 @@ char	*cp;	/* the command line argument */
  * Return the maximum amount of memory (in bytes) that this program will use.
  */
 long
-max_memory_usage()
+max_memory_usage(void)
 {
 	return((long) (N_EQUATIONS + 3L) * (long) n_tokens * sizeof(token_type) * 2L);
 }
@@ -171,7 +200,7 @@ max_memory_usage()
  * Display version info.
  */
 int
-version_report()
+version_report(void)
 {
 	long	l;
 
@@ -349,9 +378,9 @@ char	*cp;
 				}
 				subst_var_with_exp(lhs[i], &n_lhs[i], rhs[cur_equation], n_rhs[cur_equation], want.token.variable);
 				subst_var_with_exp(rhs[i], &n_rhs[i], rhs[cur_equation], n_rhs[cur_equation], want.token.variable);
+				once_through = false;
 				calc_simp(lhs[i], &n_lhs[i]);
 				calc_simp(rhs[i], &n_rhs[i]);
-				once_through = false;
 check_result:
 				contains_infinity = (exp_contains_infinity(lhs[i], n_lhs[i])
 				    || exp_contains_infinity(rhs[i], n_rhs[i]));
@@ -369,8 +398,7 @@ check_result:
 				} else {
 					if (!contains_infinity && !once_through) {
 						symb_flag = symblify;
-						simpa_repeat_side(lhs[i], &n_lhs[i], true, false);
-						simpa_repeat_side(rhs[i], &n_rhs[i], true, false);
+						simpa_repeat(i, true, false);
 						symb_flag = false;
 						once_through = true;
 						goto check_result;
@@ -487,7 +515,7 @@ enum spf_function	current_function;
 		}
 	}
 	if (!found_var(source, ns, v)) {
-		error(_("Variable not found."));
+		error(_("Specified variable not found."));
 		return false;
 	}
 	if (*cp) {
@@ -876,7 +904,8 @@ int
 push_cmd(cp)
 char	*cp;
 {
-	int	i, j, k;
+	int	start, stop;
+	int	k;
 	char	*cp1;
 
 	if (!readline_enabled) {
@@ -885,14 +914,14 @@ char	*cp;
 	}
 	do {
 		cp1 = cp;
-		if (!get_range(&cp, &i, &j)) {
+		if (!get_range(&cp, &start, &stop)) {
 			return false;
 		}
 		if (*cp && cp == cp1) {
 			error(_("Invalid argument."));
 			return false;
 		}
-		for (k = i; k <= j; k++) {
+		for (k = start; k <= stop; k++) {
 			if (n_lhs[k]) {
 				if (!push_en(k)) {
 					error(_("Push failed."));
@@ -1084,14 +1113,16 @@ char	**cpp;
  * Return true if successful.
  */
 int
-save_set_options()
+save_set_options(void)
 {
 	FILE	*fp;
+	int	pre_existing;
 
 	if (rc_file[0] == '\0') {
 		error(_("Set options startup file name not set."));
 		return false;
 	}
+	pre_existing = (access(rc_file, F_OK) == 0);
 	if ((fp = fopen(rc_file, "w")) == NULL) {
 		perror(rc_file);
 		error(_("Unable to write to set options startup file."));
@@ -1101,7 +1132,10 @@ save_set_options()
 	fprintf(fp, "; This file can be edited or deleted.\n\n");
 	output_options(fp);
 	if (fclose(fp) == 0) {
-		printf(_("All set options saved in startup file \"%s\".\n"), rc_file);
+		if (pre_existing)
+			printf(_("Startup file \"%s\" overwritten with current set options.\n"), rc_file);
+		else
+			printf(_("All current set options saved in startup file \"%s\".\n"), rc_file);
 	} else {
 		perror(rc_file);
 		error(_("Error saving set options."));
@@ -1122,31 +1156,39 @@ char	*cp;
 {
 	int	i;
 	int	negate;
-	char	*cp1, *option_string, *original_string;
+	char	*cp1, *option_string;
 
-	original_string = cp;
 try_next_param:
 	cp = skip_space(cp);
 	if (*cp == '\0') {
 		return true;
 	}
-#if	!SECURE && !LIBRARY
-	if (security_level < 3 && strncasecmp(cp, "dir", 3) == 0) {
+	if (strncasecmp(cp, "dir", 3) == 0) {
 		cp = skip_param(cp);
-		if (*cp == '\0') {
-			cp1 = getenv("HOME");
-			if (cp1) {
-				cp = cp1;
+#if	!SECURE && !LIBRARY
+		if (security_level < 3) {
+	 		if (*cp == '\0') {
+				cp1 = getenv("HOME");
+				if (cp1) {
+					cp = cp1;
+				}
 			}
+			if (chdir(cp)) {
+				error(_("Error changing directory."));
+				return false;
+			}
+			output_current_directory(stdout);
+			return true;
 		}
-		if (chdir(cp)) {
-			error(_("Error changing directory."));
-			return false;
-		}
-		output_current_directory(stdout);
-		return true;
-	}
 #endif
+#if	LIBRARY
+		error(_("Option disabled."));
+		return false;
+#else
+		error(_("Option disabled by security level."));
+		return false;
+#endif
+	}
 	negate = skip_no(&cp);
 	option_string = cp;
 	cp = skip_param(cp);
@@ -1182,6 +1224,24 @@ try_next_param:
 		}
 		return true;
 	}
+	if (strncasecmp(option_string, "rows", 3) == 0) {
+		if (negate) {
+			screen_rows = 0;
+		} else {
+			if (*cp == '\0') {
+				printf(_("Current screen rows is %d.\n"), screen_rows);
+				goto check_return;
+			}
+			i = decstrtol(cp, &cp1);
+			if (i < 0 || cp == cp1) {
+				error(_("Please specify how tall the screen is; 0 = no pagination."));
+				return false;
+			}
+			cp = cp1;
+			screen_rows = i;
+		}
+		goto check_return;
+	}
 	if (strncasecmp(option_string, "columns", 6) == 0) {
 		if (negate) {
 			screen_columns = 0;
@@ -1195,7 +1255,7 @@ try_next_param:
 			}
 			i = decstrtol(cp, &cp1);
 			if (i < 0 || cp == cp1) {
-				error(_("Please specify how wide the screen is; 0 = no column checking."));
+				error(_("Please specify how wide the screen is; 0 = no limit."));
 				return false;
 			}
 			cp = cp1;
@@ -1279,8 +1339,12 @@ try_next_param:
 		quiet_mode = negate;
 		goto try_next_param;
 	}
-#if	1
+#if	!SECURE && !LIBRARY
 	if (strncasecmp(option_string, "html", 4) == 0) {
+		if (security_level) {
+			error(_("Option disabled by security level."));
+			return false;
+		}
 		html_flag = !negate;
 		goto try_next_param;
 	}
@@ -1351,23 +1415,32 @@ try_next_param:
 		right_associative_power = !negate;
 		goto try_next_param;
 	}
+	if (strcmp_tospace(option_string, "save") == 0) {
 #if	!SECURE && !LIBRARY
-	if (security_level < 2 && strcmp_tospace(option_string, "save") == 0) {
-		if (rc_file[0] == '\0') {
-			error(_("Set options startup file name not set."));
-			return false;
-		}
-		if (negate) {
-			if (unlink(rc_file) == 0) {
-				printf(_("Set options startup file \"%s\" removed.\n"), rc_file);
-			}
-		} else {
-			if (!save_set_options())
+		if (security_level < 2) {
+			if (rc_file[0] == '\0') {
+				error(_("Set options startup file name not set."));
 				return false;
+			}
+			if (negate) {
+				if (unlink(rc_file) == 0) {
+					printf(_("Set options startup file \"%s\" removed.\n"), rc_file);
+				}
+			} else {
+				if (!save_set_options())
+					return false;
+			}
+			goto check_return;
 		}
-		goto check_return;
-	}
 #endif
+#if	LIBRARY
+		error(_("Option disabled."));
+		return false;
+#else
+		error(_("Option disabled by security level."));
+		return false;
+#endif
+	}
 	printf(_("\nCannot process set string \"%s\".\n"), option_string);
 	error(_("Unknown set option."));
 	return false;
@@ -1405,7 +1478,33 @@ int
 echo_cmd(cp)
 char	*cp;
 {
-	fprintf(gfp, "%s\n", cp);
+	int	i;
+	int	len = 0;
+	int	width, height;
+
+	if (repeat_flag) {
+		if (*cp) {
+			if (screen_columns)
+				width = screen_columns;
+			else
+				width = TEXT_COLUMNS;
+			while ((len + strlen(cp)) < width) {
+				fprintf(gfp, "%s", cp);
+				len += strlen(cp);
+			}
+			fprintf(gfp, "\n");
+		} else {
+			if (screen_rows)
+				height = screen_rows;
+			else
+				height = TEXT_ROWS;
+			for (i = 0; i < height; i++) {
+				fprintf(gfp, "\n");
+			}
+		}
+	} else {
+		fprintf(gfp, "%s\n", cp);
+	}
 	return true;
 }
 
@@ -1692,6 +1791,7 @@ calculate_cmd(cp)
 char	*cp;
 {
 	int		i, k1, k;
+	int		first, last;
 	long		v, last_v, it_v = 0;	/* Mathomatic variables */
 	long		counter, counter_max;
 	sign_array_type	sa_mark, sa_value;
@@ -1702,37 +1802,21 @@ char	*cp;
 	char		buf[MAX_CMD_LEN];
 	int		factor_flag = false, value_entered;
 
-	if (current_not_defined()) {
-		return false;
-	}
 	for (;; cp = skip_param(cp)) {
 		if (strcmp_tospace(cp, "factor") == 0) {
 			factor_flag = true;
 			continue;
 		}
-		if (strcmp_tospace(cp, "repeat") == 0) {
-			repeat_flag = true;
-			continue;
-		}
 		break;
 	}
-	i = cur_equation;
-	if (n_rhs[i]) {
-		source = rhs[i];
-		n = n_rhs[i];
-	} else {
-		source = lhs[i];
-		n = n_lhs[i];
-	}
-	if (*cp) {
+	if (!get_range(&cp, &first, &last)) {
+		return false;
+	} 
+	if (first == last && *cp) {
 		cp = parse_var2(&it_v, cp);
 		if (cp == NULL) {
 			return false;
 		}
-		if (!found_var(source, n, it_v)) {
-			error(_("Specified iteration variable not found."));
-			return false;
-		}
 		if (*cp == '\0') {
 			my_strlcpy(prompt_str, _("Enter maximum number of iterations: "), sizeof(prompt_str));
 			if ((cp = get_string(buf, sizeof(buf))) == NULL)
@@ -1748,144 +1832,165 @@ char	*cp;
 			iterations = LONG_MAX - 1L;
 		}
 	}
+	if (extra_characters(cp)) {
+		return false;
+	}
 calc_again:
 	value_entered = false;
-	n_trhs = n;
-	blt(trhs, source, n_trhs * sizeof(token_type));
-	last_v = 0;
-	for (;;) {
-		v = -1;
-		for (k1 = 0; k1 < n; k1 += 2) {
-			if (source[k1].kind == VARIABLE) {
-				if (source[k1].token.variable > last_v
-				    && (v == -1 || source[k1].token.variable < v))
-					v = source[k1].token.variable;
+	for (i = first; i <= last; i++) {
+		if (n_rhs[i]) {
+			source = rhs[i];
+			n = n_rhs[i];
+		} else {
+			source = lhs[i];
+			n = n_lhs[i];
+		}
+		if (it_v) {
+			if (!found_var(source, n, it_v)) {
+				error(_("Specified iteration variable not found."));
+				return false;
 			}
 		}
-		if (v == -1)
-			break;
-		last_v = v;
-		if ((v & VAR_MASK) <= SIGN || v == it_v) {
-			continue;
-		}
-		list_var(v, 0);
-		snprintf(prompt_str, sizeof(prompt_str), _("Enter %s: "), var_str);
-		if (!get_expr(tlhs, &n_tlhs)) {
-			continue;
-		}
-		value_entered = true;
-		/* Disguise all variables in the entered expression by making them negative. */
-		/* That way they won't be improperly substituted in the future. */
-		for (k1 = 0; k1 < n_tlhs; k1 += 2)
-			if (tlhs[k1].kind == VARIABLE)
-				tlhs[k1].token.variable = -tlhs[k1].token.variable;
-		subst_var_with_exp(trhs, &n_trhs, tlhs, n_tlhs, v);
-	}
-	/* Restore disguised variables: */
-	for (k1 = 0; k1 < n_trhs; k1 += 2)
-		if (trhs[k1].kind == VARIABLE && trhs[k1].token.variable < 0)
-			trhs[k1].token.variable = -trhs[k1].token.variable;
-	if (it_v) {
-		/* Handle the iteration option, where the simplified result is repeatedly plugged into variable it_v. */
-		list_var(it_v, 0);
-		snprintf(prompt_str, sizeof(prompt_str), _("Enter initial %s: "), var_str);
-		while (!get_expr(tes, &n_tes)) {
-			return repeat_flag;
-		}
-		value_entered = true;
-		calc_simp(tes, &n_tes);
-		blt(tlhs, trhs, n_trhs * sizeof(token_type));
-		n_tlhs = n_trhs;
-		for (l = 0; l < iterations; l++) {
-			side_debug(1, tes, n_tes);
-			blt(trhs, tlhs, n_tlhs * sizeof(token_type));
-			n_trhs = n_tlhs;
-			subst_var_with_exp(trhs, &n_trhs, tes, n_tes, it_v);
-			calc_simp(trhs, &n_trhs);
-			if (se_compare(trhs, n_trhs, tes, n_tes, &diff_sign) && !diff_sign) {
-				fprintf(gfp, _("Convergence reached after %ld iterations.\n"), l + 1);
+		n_trhs = n;
+		blt(trhs, source, n_trhs * sizeof(token_type));
+		last_v = 0;
+		for (;;) {
+			v = -1;
+			for (k1 = 0; k1 < n; k1 += 2) {
+				if (source[k1].kind == VARIABLE) {
+					if (source[k1].token.variable > last_v
+					    && (v == -1 || source[k1].token.variable < v))
+						v = source[k1].token.variable;
+				}
+			}
+			if (v == -1)
 				break;
+			last_v = v;
+			if ((v & VAR_MASK) <= SIGN || v == it_v) {
+				continue;
 			}
-			blt(tes, trhs, n_trhs * sizeof(token_type));
-			n_tes = n_trhs;
+			list_var(v, 0);
+			snprintf(prompt_str, sizeof(prompt_str), _("Enter %s: "), var_str);
+			if (!get_expr(tlhs, &n_tlhs)) {
+				continue;
+			}
+			value_entered = true;
+			/* Disguise all variables in the entered expression by making them negative. */
+			/* That way they won't be improperly substituted in the future. */
+			for (k1 = 0; k1 < n_tlhs; k1 += 2)
+				if (tlhs[k1].kind == VARIABLE)
+					tlhs[k1].token.variable = -tlhs[k1].token.variable;
+			subst_var_with_exp(trhs, &n_trhs, tlhs, n_tlhs, v);
 		}
-	}
-	calc_simp(trhs, &n_trhs);
+		/* Restore disguised variables: */
+		for (k1 = 0; k1 < n_trhs; k1 += 2)
+			if (trhs[k1].kind == VARIABLE && trhs[k1].token.variable < 0)
+				trhs[k1].token.variable = -trhs[k1].token.variable;
+		if (it_v) {
+			/* Handle the iteration option, where the simplified result is repeatedly plugged into variable it_v. */
+			list_var(it_v, 0);
+			snprintf(prompt_str, sizeof(prompt_str), _("Enter initial %s: "), var_str);
+			while (!get_expr(tes, &n_tes)) {
+				printf("Aborted.\n");
+				return repeat_flag;
+			}
+			value_entered = true;
+			calc_simp(tes, &n_tes);
+			blt(tlhs, trhs, n_trhs * sizeof(token_type));
+			n_tlhs = n_trhs;
+			for (l = 0; l < iterations; l++) {
+				side_debug(1, tes, n_tes);
+				blt(trhs, tlhs, n_tlhs * sizeof(token_type));
+				n_trhs = n_tlhs;
+				subst_var_with_exp(trhs, &n_trhs, tes, n_tes, it_v);
+				calc_simp(trhs, &n_trhs);
+				if (se_compare(trhs, n_trhs, tes, n_tes, &diff_sign) && !diff_sign) {
+					fprintf(gfp, _("Convergence reached after %ld iterations.\n"), l + 1);
+					break;
+				}
+				blt(tes, trhs, n_trhs * sizeof(token_type));
+				n_tes = n_trhs;
+			}
+		}
+		calc_simp(trhs, &n_trhs);
 
-	/* Now substitute all sign variables with +1 and -1. */
-	CLEAR_ARRAY(sa_mark);
-	for (k1 = 0; k1 < n_trhs; k1 += 2) {
-		if (trhs[k1].kind == VARIABLE && (trhs[k1].token.variable & VAR_MASK) == SIGN) {
-			sa_mark[(trhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+		/* Now substitute all sign variables with +1 and -1. */
+		CLEAR_ARRAY(sa_mark);
+		for (k1 = 0; k1 < n_trhs; k1 += 2) {
+			if (trhs[k1].kind == VARIABLE && (trhs[k1].token.variable & VAR_MASK) == SIGN) {
+				sa_mark[(trhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK] = true;
+			}
 		}
-	}
-	for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
-		if (sa_mark[k1]) {
-			k++;
-		}
-	}
-	counter_max = (1L << k) - 1L;
-	if (counter_max) {
-		fprintf(gfp, _("There are %ld solutions.\n"), counter_max + 1);
-	}
-	for (counter = 0; counter <= counter_max; counter++) {
-		blt(tlhs, trhs, n_trhs * sizeof(token_type));
-		n_tlhs = n_trhs;
 		for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
 			if (sa_mark[k1]) {
-				sa_value[k1] = (((1L << k) & counter) != 0);
 				k++;
 			}
 		}
-		for (k1 = 0; k1 < n_tlhs; k1 += 2) {
-			if (tlhs[k1].kind == VARIABLE && (tlhs[k1].token.variable & VAR_MASK) == SIGN) {
-				if (sa_value[(tlhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
-					tlhs[k1].kind = CONSTANT;
-					tlhs[k1].token.constant = -1.0;
-				} else {
-					tlhs[k1].kind = CONSTANT;
-					tlhs[k1].token.constant = 1.0;
+		counter_max = (1L << k) - 1L;
+		if (counter_max) {
+			fprintf(gfp, _("There are %ld solutions.\n"), counter_max + 1);
+		}
+		for (counter = 0; counter <= counter_max; counter++) {
+			blt(tlhs, trhs, n_trhs * sizeof(token_type));
+			n_tlhs = n_trhs;
+			for (k1 = 0, k = 0; k1 < ARR_CNT(sa_mark); k1++) {
+				if (sa_mark[k1]) {
+					sa_value[k1] = (((1L << k) & counter) != 0);
+					k++;
 				}
 			}
-		}
-		for (k1 = 0, k = false; k1 < ARR_CNT(sa_mark); k1++) {
-			if (sa_mark[k1]) {
-				if (k) {
-					fprintf(gfp, ", ");
-				} else {
-					fprintf(gfp, _("\nSolution number %ld with "), counter + 1);
+			for (k1 = 0; k1 < n_tlhs; k1 += 2) {
+				if (tlhs[k1].kind == VARIABLE && (tlhs[k1].token.variable & VAR_MASK) == SIGN) {
+					if (sa_value[(tlhs[k1].token.variable >> VAR_SHIFT) & SUBSCRIPT_MASK]) {
+						tlhs[k1].kind = CONSTANT;
+						tlhs[k1].token.constant = -1.0;
+					} else {
+						tlhs[k1].kind = CONSTANT;
+						tlhs[k1].token.constant = 1.0;
+					}
 				}
-				list_var((long) SIGN + (((long) k1) << VAR_SHIFT), 0);
-				fprintf(gfp, "%s = ", var_str);
-				if (sa_value[k1]) {
-					fprintf(gfp, "-1");
-				} else {
-					fprintf(gfp, "1");
-				}
-				k = true;
 			}
-		}
-		if (k)
-			fprintf(gfp, ":\n");
-		calc_simp(tlhs, &n_tlhs);
-		if (factor_flag) {
-			mid_simp_side(tlhs, &n_tlhs);
-		}
-		fprintf(gfp, " ");
-		if (n_rhs[i]) {
-			list_proc(lhs[i], n_lhs[i], false);
-			fprintf(gfp, " = ");
-		}
-		list_factor(tlhs, &n_tlhs, factor_flag);
-		if (n_tlhs <= 9 && make_fractions(tlhs, &n_tlhs)) {
-			group_proc(tlhs, &n_tlhs);
-			fprintf(gfp, " = ");
+			for (k1 = 0, k = false; k1 < ARR_CNT(sa_mark); k1++) {
+				if (sa_mark[k1]) {
+					if (k) {
+						fprintf(gfp, ", ");
+					} else {
+						fprintf(gfp, _("\nSolution number %ld with "), counter + 1);
+					}
+					list_var((long) SIGN + (((long) k1) << VAR_SHIFT), 0);
+					fprintf(gfp, "%s = ", var_str);
+					if (sa_value[k1]) {
+						fprintf(gfp, "-1");
+					} else {
+						fprintf(gfp, "1");
+					}
+					k = true;
+				}
+			}
+			if (k)
+				fprintf(gfp, ":\n");
+			calc_simp(tlhs, &n_tlhs);
+			if (factor_flag) {
+				mid_simp_side(tlhs, &n_tlhs);
+			}
+			fprintf(gfp, " ");
+			if (n_rhs[i]) {
+				list_proc(lhs[i], n_lhs[i], false);
+				fprintf(gfp, " = ");
+			}
 			list_factor(tlhs, &n_tlhs, factor_flag);
+			if (n_tlhs <= 9 && make_fractions(tlhs, &n_tlhs)) {
+				group_proc(tlhs, &n_tlhs);
+				fprintf(gfp, " = ");
+				list_factor(tlhs, &n_tlhs, factor_flag);
+			}
+			fprintf(gfp, "\n");
 		}
-		fprintf(gfp, "\n");
 	}
-	if (value_entered && repeat_flag)
+	if (value_entered && repeat_flag) {
+		fprintf(gfp, "Repeating:\n");
 		goto calc_again;
+	}
 	return true;
 }
 #endif
@@ -2117,11 +2222,15 @@ char	*cp;
 			goto times_neg1;
 		}
 	}
-	debug_string(0, _("Solving both equations for zero and unfactoring..."));
+	debug_string(0, _("Solving both equations for zero and expanding..."));
 	if (solve_sub(&zero_token, 1, lhs[i], &n_lhs[i], rhs[i], &n_rhs[i]) <= 0)
 		return false;
 	if (solve_sub(&zero_token, 1, lhs[j], &n_lhs[j], rhs[j], &n_rhs[j]) <= 0)
 		return false;
+	if (compare_rhs(i, j, &diff_sign)) {
+		fprintf(gfp, _("Equations are identical.\n"));
+		return true;
+	}
 	uf_simp(rhs[i], &n_rhs[i]);
 	uf_simp(rhs[j], &n_rhs[j]);
 	if (compare_rhs(i, j, &diff_sign)) {
@@ -2225,25 +2334,42 @@ char	*cp;
 	int		nleft = 0, nright = 0;
 	double		lcm, d1, d2, d3, d4, d5;
 	complexs	c1, c2, c3;
+	char		*cp_start;
 
+	cp_start = cp;
 	pull_number = -1;	/* Operands are last two entered expressions when using library. */
-	if (*cp) {
+	if (*cp && isvarchar(*cp)) {
 		cp = parse_var2(&v, cp);
 		if (cp == NULL) {
 			return false;
 		}
-		if (extra_characters(cp))
-			return false;
+		fprintf(gfp, _("You have entered a base variable.\n"));
 	}
 	i = next_espace();
-do_repeat:
+	if (*cp) {
+		input_column += (cp - cp_start);
+		cp = parse_expr(rhs[i], &nright, cp, false);
+		if (cp == NULL || nright <= 0) {
+			return false;
+		}
+	}
+	if (*cp) {
+		cp_start = cp;
+		cp = skip_space(cp);
+		input_column += (cp - cp_start);
+		cp = parse_expr(lhs[i], &nleft, cp, false);
+		if (cp == NULL || extra_characters(cp) || nleft <= 0) {
+			return false;
+		}
+	}
+do_repeat_prompt:
 /* prompt for the two operands */
 	my_strlcpy(prompt_str, _("Enter dividend: "), sizeof(prompt_str));
-	if (!get_expr(rhs[i], &nright)) {
+	if (nright == 0 && !get_expr(rhs[i], &nright)) {
 		return repeat_flag;
 	}
 	my_strlcpy(prompt_str, _("Enter divisor: "), sizeof(prompt_str));
-	if (!get_expr(lhs[i], &nleft)) {
+	if (nleft == 0 && !get_expr(lhs[i], &nleft)) {
 		return repeat_flag;
 	}
 	fprintf(gfp, "\n");
@@ -2343,6 +2469,11 @@ do_repeat:
 	if (repeat_flag)
 		goto do_repeat;
 	return true;
+
+do_repeat:
+	nright = 0;
+	nleft = 0;
+	goto do_repeat_prompt;
 }
 
 /*
@@ -2364,10 +2495,6 @@ char	*cp;
 	if (current_not_defined()) {
 		return false;
 	}
-	if (strcmp_tospace(cp, "repeat") == 0) {
-		repeat_flag = true;
-		cp = skip_param(cp);
-	}
 	if (*cp == '\0') {
 		my_strlcpy(prompt_str, _("Enter variables to eliminate: "), sizeof(prompt_str));
 		cp = get_string(buf, sizeof(buf));
@@ -2427,6 +2554,12 @@ next_var:
 			}
 		}
 		if (did_something) {
+#if	0
+			simpa_side(lhs[cur_equation], &n_lhs[cur_equation], false, false);
+			if (n_rhs[cur_equation] > 0) {
+				simpa_side(rhs[cur_equation], &n_rhs[cur_equation], false, false);
+			}
+#endif
 			did_something = return_result(cur_equation);
 		} else {
 			error(_("No substitutions made."));
@@ -2480,7 +2613,7 @@ next_var:
 
 /*
  * Solve equation number i for v and substitute the RHS
- * into all occurrences of v in the current equation, simplifying a little.
+ * into all occurrences of v in the current equation, then simplify.
  */
 static int
 elim_sub(i, v)
@@ -2515,6 +2648,7 @@ long	v;	/* Mathomatic variable */
 	}
 	subst_var_with_exp(rhs[cur_equation], &n_rhs[cur_equation], rhs[i], n_rhs[i], v);
 	subst_var_with_exp(lhs[cur_equation], &n_lhs[cur_equation], rhs[i], n_rhs[i], v);
+
 	simp_equation(cur_equation);
 	return true;
 }
@@ -2578,6 +2712,11 @@ char	*cp;
 				if (result_str == NULL)
 					result_str = list_equation(i, false);
 				result_en = i;
+				if (gfp != stdout) {
+					if (flist_equation(i) > 0) {
+						displayed = true;
+					}
+				}
 #else
 				if (flist_equation(i) > 0) {
 					displayed = true;
@@ -2698,7 +2837,7 @@ char	*cp;
 			language = PYTHON;
 			continue;
 		}
-		if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
+		if (strncasecmp(cp, "integer", 3) == 0) {
 			int_flag = true;
 			continue;
 		}
@@ -2780,7 +2919,8 @@ int
 variables_cmd(cp)
 char	*cp;
 {
-	int			i, j, k;
+	int			start, stop;
+	int			k;
 	int			i1;
 	long			v1, last_v;		/* Mathomatic variables */
 	int			vc, cnt;		/* variable counts */
@@ -2788,7 +2928,7 @@ char	*cp;
 	token_type		*p1;
 	int			n1;
 	enum language_list	lang_code = 0;		/* default to no programming language */
-	int			int_flag = false, imag_flag = false;
+	int			int_flag = false, imag_flag = false, count_flag = false;
 	char			imag_array[N_EQUATIONS];
 	char			*range_start, *cp1;
 	int			rv = true;
@@ -2800,22 +2940,25 @@ char	*cp;
 	} else if (strcmp_tospace(cp, "java") == 0) {
 		cp = skip_param(cp);
 		lang_code = JAVA;
-	} else if (strcmp_tospace(cp, "int") == 0 || strcmp_tospace(cp, "integer") == 0) {
+	} else if (strncasecmp(cp, "integer", 3) == 0) {
 		cp = skip_param(cp);
 		lang_code = C;
 		int_flag = true;
+	} else if (strncasecmp(cp, "counts", 5) == 0) {
+		cp = skip_param(cp);
+		count_flag = true;
 	}
 	range_start = cp;
 	do {
 		cp1 = cp;
-		if (!get_range(&cp, &i, &j)) {
+		if (!get_range(&cp, &start, &stop)) {
 			return false;
 		}
 		if (*cp && cp == cp1) {
 			error(_("Invalid argument."));
 			return false;
 		}
-		for (k = i; k <= j; k++) {
+		for (k = start; k <= stop; k++) {
 			if (n_lhs[k] <= 0)
 				continue;
 			if (n_rhs[k] > 0) {
@@ -2846,7 +2989,7 @@ char	*cp;
 		cp = range_start;
 		do {
 			cp1 = cp;
-			if (!get_range(&cp, &i, &j)) {
+			if (!get_range(&cp, &start, &stop)) {
 				return false;
 			}
 #if	DEBUG
@@ -2854,7 +2997,7 @@ char	*cp;
 				error_bug("Bug in variables command.");
 			}
 #endif
-			for (k = i; k <= j; k++) {
+			for (k = start; k <= stop; k++) {
 				if (n_lhs[k] <= 0)
 					continue;
 				p1 = lhs[k];
@@ -2916,6 +3059,9 @@ char	*cp;
 				fprintf(gfp, "double %s;\n", var_str);
 			}
 		} else {
+			if (count_flag) {
+				fprintf(gfp, "%6d ", va[i1].count);
+			}
 			fprintf(gfp, "%s\n", var_str);
 		}
 	}
@@ -2941,25 +3087,26 @@ int
 approximate_cmd(cp)
 char	*cp;
 {
-	int	i, j;
+	int	start, stop;
+	int	k;
 	char	*cp1;
 
 	do {
 		cp1 = cp;
-		if (!get_range(&cp, &i, &j)) {
+		if (!get_range(&cp, &start, &stop)) {
 			return false;
 		}
 		if (*cp && cp == cp1) {
 			error(_("Invalid argument."));
 			return false;
 		}
-		for (; i <= j; i++) {
-			if (n_lhs[i]) {
-				approximate(lhs[i], &n_lhs[i]);
-				if (n_rhs[i]) {
-					approximate(rhs[i], &n_rhs[i]);
+		for (k = start; k <= stop; k++) {
+			if (n_lhs[k]) {
+				approximate(lhs[k], &n_lhs[k]);
+				if (n_rhs[k]) {
+					approximate(rhs[k], &n_rhs[k]);
 				}
-				if (!return_result(i)) {
+				if (!return_result(k)) {
 					return false;
 				}
 			}
@@ -3088,11 +3235,13 @@ int
 simplify_cmd(cp)
 char	*cp;
 {
-	int		i, i1, j;
+	int		i, i1;
+	int		first, last;
 	int		k, k1;
 	long		counter, counter_max, previous_solution_number[N_EQUATIONS];
 	sign_array_type	sa_mark, sa_value;
 	int		sign_flag = false, quick_flag = false, quickest_flag = false, symb = false, frac_flag = false;
+	char		*cp1;
 
 	for (;; cp = skip_param(cp)) {
 		if (strncasecmp(cp, "sign", 4) == 0) {
@@ -3117,17 +3266,25 @@ char	*cp;
 		}
 		break;
 	}
-	if (!get_range_eol(&cp, &i, &j)) {
-		return false;
-	}
 	symb_flag = symb;
-	for (; i <= j; i++) {
-		if (n_lhs[i]) {
+	do {
+		cp1 = cp;
+		if (!get_range(&cp, &first, &last)) {
+			symb_flag = false;
+			return false;
+		}
+		if (*cp && cp == cp1) {
+			symb_flag = false;
+			error(_("Invalid argument."));
+			return false;
+		}
+		for (i = first; i <= last; i++) {
+			if (n_lhs[i] <= 0)
+				continue;
 			if (quickest_flag) {
 				simp_equation(i);
 			} else {
-				simpa_repeat_side(lhs[i], &n_lhs[i], quick_flag, frac_flag);
-				simpa_repeat_side(rhs[i], &n_rhs[i], quick_flag, frac_flag);
+				simpa_repeat(i, quick_flag, frac_flag);
 			}
 			if (!return_result(i)) {
 				symb_flag = false;
@@ -3209,8 +3366,7 @@ char	*cp;
 				if (quickest_flag) {
 					simp_equation(i1);
 				} else {
-					simpa_repeat_side(lhs[i1], &n_lhs[i1], quick_flag, frac_flag);
-					simpa_repeat_side(rhs[i1], &n_rhs[i1], quick_flag, frac_flag);
+					simpa_repeat(i1, quick_flag, frac_flag);
 				}
 				for (k1 = 0; k1 < ARR_CNT(previous_solution_number); k1++) {
 					if (previous_solution_number[k1]) {
@@ -3228,7 +3384,7 @@ char	*cp;
 				}
 			}
 		}
-	}
+	} while (*cp);
 	symb_flag = false;
 	return true;
 }
@@ -3378,7 +3534,7 @@ try_parsing:
 					}
 				}
 				if (!found) {
-					warning(_("Variable not found."));
+					warning(_("Specified variable not found."));
 				}
 			}
 			for (i1 = i; i1 <= j; i1++) {
@@ -3415,7 +3571,7 @@ unfactor_cmd(cp)
 char	*cp;
 {
 	int	i, j, k;
-	int	quick_flag = false, fraction_flag = false, power_flag = false;
+	int	quick_flag = false, fraction_flag = false, power_flag = false, count_flag = false;
 
 	for (;; cp = skip_param(cp)) {
 		if (strncasecmp(cp, "quick", 4) == 0) {
@@ -3430,6 +3586,10 @@ char	*cp;
 			power_flag = true;
 			continue;
 		}
+		if (strncasecmp(cp, "count", 4) == 0) {
+			count_flag = true;
+			continue;
+		}
 		break;
 	}
 	if (!get_range_eol(&cp, &i, &j)) {
@@ -3458,6 +3618,14 @@ char	*cp;
 				partial_flag = true;
 				return false;
 			}
+			if (count_flag) {
+				if (n_rhs[k]) {
+					fprintf(gfp, _("#%d: LHS consists of %d terms; "), k + 1, level1_plus_count(lhs[k], n_lhs[k]) + 1);
+					fprintf(gfp, _("RHS consists of %d terms.\n"), level1_plus_count(rhs[k], n_rhs[k]) + 1);
+				} else {
+					fprintf(gfp, _("#%d: Expression consists of %d terms.\n"), k + 1, level1_plus_count(lhs[k], n_lhs[k]) + 1);
+				}
+			}
 		}
 	} else {
 		for (k = i; k <= j; k++) {
@@ -3478,6 +3646,14 @@ char	*cp;
 				partial_flag = true;
 				return false;
 			}
+			if (count_flag) {
+				if (n_rhs[k]) {
+					fprintf(gfp, _("#%d: LHS consists of %d terms; "), k + 1, level1_plus_count(lhs[k], n_lhs[k]) + 1);
+					fprintf(gfp, _("RHS consists of %d terms.\n"), level1_plus_count(rhs[k], n_rhs[k]) + 1);
+				} else {
+					fprintf(gfp, _("#%d: Expression consists of %d terms.\n"), k + 1, level1_plus_count(lhs[k], n_lhs[k]) + 1);
+				}
+			}
 		}
 	}
 	partial_flag = true;
