@@ -1,21 +1,24 @@
 /*
  * Floating point complex number routines specifically for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
@@ -116,6 +119,9 @@ do_repeat:
 			}
 			fprintf(gfp, "\n");
 #if	!SILENT
+			if (debug_level < 0) {
+				continue;
+			}
 			check = c2;
 			for (d = 1.0; d < root; d += 1.0) {
 				check = complex_mult(check, c2);
@@ -143,7 +149,7 @@ do_repeat:
  * This only gives one root, even when there may be many roots.
  * Works best when the equation side has been approximated before this.
  *
- * Return true and display a warning message if the equation side was modified.
+ * Return true if the equation side was modified.
  */
 int
 complex_root_simp(equation, np)
@@ -216,15 +222,23 @@ start_over:
 
 /*
  * Approximate all roots of complex numbers in an equation side.
+ *
+ * Return true if anything was approximated.
  */
-void
+int
 approximate_complex_roots(equation, np)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
-	do {
+	int	rv = false;
+
+	for (;;) {
 		elim_loop(equation, np);
-	} while (complex_root_simp(equation, np));
+		if (!complex_root_simp(equation, np))
+			break;
+		rv = true;
+	}
+	return rv;
 }
 
 /*
