@@ -373,7 +373,7 @@ char	*cp;
 		return false;
 	}
 	if (v != MATCH_ANY && !found_var(source, *nps, v)) {
-		warning(_("Variable not found; the derivative will be zero."));
+		warning(_("Specified variable not found; the derivative will be zero."));
 	}
 #if	!SILENT
 	if (debug_level >= 0) {
@@ -396,12 +396,19 @@ char	*cp;
 	n1 = *nps;
 /* do the actual differentiating and simplifying: */
 	for (l1 = 0; l1 < order; l1++) {
+		if (order != 1) {
+			if (n1 == 1 && dest[0].kind == CONSTANT && dest[0].token.constant == 0.0) {
+				fprintf(gfp, _("0 reached after %ld derivatives taken.\n"), l1);
+				order = l1;
+				break;
+			}
+		}
 		if (!differentiate(dest, &n1, v)) {
 			error(_("Differentiation failed."));
 			return false;
 		}
 		if (simplify_flag) {
-			simpa_side(dest, &n1, true, false);
+			simpa_repeat_side(dest, &n1, true, false);
 		} else {
 			elim_loop(dest, &n1);
 		}
@@ -416,7 +423,9 @@ char	*cp;
 				var_str[len++] = '\'';
 			}
 			var_str[len] = '\0';
-			parse_var(&lhs[i][0].token.variable, var_str);
+			if (l1 == order) {
+				parse_var(&lhs[i][0].token.variable, var_str);
+			}
 		}
 	}
 	cur_equation = i;
@@ -480,7 +489,7 @@ char	*cp;
 		}
 	}
 	if (!found_var(source, n, v)) {
-		error(_("Variable not found; the derivative would be zero."));
+		error(_("Specified variable not found; the derivative would be zero."));
 		return false;
 	}
 	blt(rhs[i], source, n * sizeof(token_type));
@@ -490,7 +499,7 @@ char	*cp;
 			error(_("Differentiation failed."));
 			return false;
 		}
-		simpa_side(rhs[i], &n, true, false);
+		simpa_repeat_side(rhs[i], &n, true, false);
 	}
 	if (!found_var(rhs[i], n, v)) {
 		error(_("There are no solutions."));
@@ -509,7 +518,7 @@ char	*cp;
 		error(_("Solve failed."));
 		return false;
 	}
-	simpa_side(rhs[i], &n_rhs[i], false, false);
+	simpa_repeat_side(rhs[i], &n_rhs[i], false, false);
 	return return_result(cur_equation);
 }
 
@@ -584,7 +593,7 @@ char	*cp;
 		}
 	}
 	if (!found_var(source, *nps, v)) {
-		warning(_("Variable not found."));
+		warning(_("Specified variable not found."));
 	}
 	blt(rhs[our], source, *nps * sizeof(token_type));
 	our_nrhs = *nps;
@@ -717,7 +726,7 @@ loop_again:
 			}
 		}
 		symb_flag = symblify;
-		simpa_side(rhs[our], &our_nrhs, true, true);
+		simpa_repeat_side(rhs[our], &our_nrhs, true, true);
 		symb_flag = false;
 		if (our_nrhs != 1 || rhs[our][0].kind != CONSTANT || rhs[our][0].token.constant != 0.0) {
 			i1 = n1;
@@ -795,7 +804,7 @@ char	*cp;
 		}
 	}
 	if (!found_var(rhs[cur_equation], n_rhs[cur_equation], v)) {
-		error(_("Variable not found."));
+		error(_("Limit variable not found."));
 		return false;
 	}
 	if (*cp == '=') {
