@@ -124,7 +124,7 @@ double	arg;
  * Nothing is changed and the value is returned.
  */
 double
-multiply_out_unique()
+multiply_out_unique(void)
 {
 	int	i, j;
 	double	d;
@@ -144,7 +144,7 @@ multiply_out_unique()
  * to fill out unique[] with the prime factors of value.
  */
 void
-display_unique()
+display_unique(void)
 {
 	int	i;
 	double	value;
@@ -170,7 +170,7 @@ display_unique()
  * Return true if x is a prime number.
  */
 int
-is_prime()
+is_prime(void)
 {
 	double	value;
 
