@@ -40,10 +40,10 @@ token_type	*lhs[N_EQUATIONS],		/* The Left Hand Sides of equation spaces */
 int		n_lhs[N_EQUATIONS],		/* number of tokens in each lhs[], 0 means equation space is empty */
 		n_rhs[N_EQUATIONS];		/* number of tokens in each rhs[], 0 means not an equation */
 
-token_type	*tlhs,				/* LHS during solve and temporary storage for expressions */
-		*trhs,				/* RHS during solve and temporary storage for expressions */
-		*tes,				/* temporary equation side, used only in commands */
-		*scratch;			/* very temporary storage for expressions, used in low level routines */
+token_type	*tlhs,				/* LHS during solve and temporary storage for expressions, quotient for poly_div() and smart_div(). */
+		*trhs,				/* RHS during solve and temporary storage for expressions, remainder for poly_div() and smart_div(). */
+		*tes,				/* temporary equation side, used in commands, simpa_repeat_side(), simple_frac_repeat_side(), etc. */
+		*scratch;			/* very temporary storage for expressions, used only in low level routines for expression manipulation. */
 
 int		n_tlhs,				/* number of tokens in tlhs */
 		n_trhs,				/* number of tokens in trhs */
@@ -146,7 +146,7 @@ int		pull_number;		/* equation space number to pull when using the library */
 #if	!SECURE
 int		security_level;		/* current enforced security level for session, -1 for m4 Mathomatic */
 #endif
-int		repeat_flag;		/* true if the command is to repeat its function, set by repeat command */
+int		repeat_flag;		/* true if the command is to repeat its function or simplification, set by repeat command */
 int		show_usage;		/* show command usage info if a command fails and this flag is true */
 
 /* library variables go here */
@@ -154,7 +154,7 @@ char		*result_str;		/* returned result text string when using as library */
 int		result_en = -1;		/* equation number of the returned result, if stored in an equation space */
 const char	*error_str;		/* last error string */
 const char	*warning_str;		/* last warning string */
-#if	LIBRARY
-/* Screen character array, for buffering library, page-at-a-time, string output: */
-char vscreen[TEXT_ROWS][TEXT_COLUMNS];
-#endif
+
+/* Screen character array, for buffering page-at-a-time string output: */
+char		*vscreen[TEXT_ROWS];
+int		current_columns;
