@@ -30,8 +30,6 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 #define	CMD_REQUIRED_NCHARS	4	/* Only type this many characters to run a Mathomatic command. */
 					/* Set this to a high number like 50 to require all letters of a command to be typed. */
 
-#define	REPEAT_CMD_STR	"repeat"	/* Name of the repeat command.  This command is specially handled and not listed in the table below. */
-
 /*
  * The following structure is used for each Mathomatic command.
  */
@@ -52,16 +50,16 @@ static com_type com_list[] = {
 /*	command name,	alternate name,		function,		usage,							information */
 {	"approximate",	NULL,			approximate_cmd,	"[equation-number-ranges]",				"Approximate all numerical values in equation spaces.", "Related command: calculate" },
 #if	!LIBRARY
-{	"calculate",	NULL,			calculate_cmd,		"[\"factor\"] [feedback-variable number-of-iterations]",	"Temporarily plug in values for variables and approximate.", "This command may be preceded with \"repeat\"." },
+{	"calculate",	NULL,			calculate_cmd,		"[\"factor\"] [equation-number-range] [variable iterations]",	"Temporarily plug in values for variables and approximate.", "This command may be preceded with \"repeat\"." },
 #endif
 {	"clear",	NULL,			clear_cmd,		"[equation-number-ranges]",				"Delete expressions stored in memory so equation spaces can be reused.", "Tip: Use \"clear all\" to quickly restart Mathomatic." },
 {	"code",		NULL,			code_cmd,		"[\"c\" or \"java\" or \"python\" or \"integer\"] [equation-number-ranges]",	"Output C, Java, or Python code for the specified equations.", "Related commands: simplify, optimize, and variables" },
-{	"compare",	NULL,			compare_cmd,		"[\"symbolic\" \"approx\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces to see if mathematically the same." },
+{	"compare",	NULL,			compare_cmd,		"[\"symbolic\" \"approx\"] equation-number [\"with\" equation-number]",	"Compare two equation spaces for mathematical equivalence.", "This command may be preceded with \"repeat\"." },
 {	"copy",		NULL,			copy_cmd,		"[equation-number-ranges]",				"Duplicate the contents of the specified equation spaces." },
 {	"derivative",	"differentiate",	derivative_cmd,		"[\"nosimplify\"] variable or \"all\" [order]",		"Symbolically differentiate and simplify, order times." },
 {	"display",	NULL,			display_cmd,		"[\"factor\" \"mixed\"] [equation-number-ranges]",	"Display equation spaces in pretty, 2D multi-line fraction format." },
-{	"divide",	NULL,			divide_cmd,		"[variable]",						"Prompt for 2 numbers or polynomials and divide. Give result and GCD.", "This command may be preceded with \"repeat\"." },
-{	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline." },
+{	"divide",	NULL,			divide_cmd,		"[base-variable] [dividend divisor]",			"Divide 2 numbers or polynomials. Give detailed result and GCD.", "This command may be preceded with \"repeat\"." },
+{	"echo",		NULL,			echo_cmd,		"[text]",						"Output a line of text, followed by a newline.", "This command may be preceded with \"repeat\"." },
 #if	SHELL_OUT
 {	"edit",		NULL,			edit_cmd,		"[file-name]",						"Edit all equation spaces or an input file, then read them in.", "Editor name in EDITOR environment variable." },
 #endif
@@ -78,11 +76,11 @@ static com_type com_list[] = {
 {	"laplace",	NULL,			laplace_cmd,		"[\"inverse\"] variable",				"Compute the Laplace or inverse Laplace transform of polynomials.", "This command only works with polynomials." },
 {	"limit",	NULL,			limit_cmd,		"variable expression",					"Take the limit as variable goes to expression.", "This limit command is experimental." },
 {	"list",		NULL,			list_cmd,		"[\"export\" or \"maxima\" or \"gnuplot\" or \"hex\"] [equation-number-ranges]",	"Display equation spaces in single-line (one-dimensional) format.", "Options to export expressions to other math programs." },
-{	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions]",		"Do numerical definite integration using Simpson's rule.", "This command cannot integrate over singularities." },
+{	"nintegrate",	NULL,			nintegrate_cmd,		"[\"trapezoid\"] variable [partitions [lower-bound upper-bound]]",	"Do numerical definite integration using Simpson's rule.", "This command cannot integrate over singularities." },
 {	"optimize",	NULL,			optimize_cmd,		"[equation-number-range]",				"Split up equations into smaller, more efficient equations." },
 {	"pause",	NULL,			pause_cmd,		"[text]",						"Display a line of text and wait for user to press the Enter key." },
 #if	SHELL_OUT
-{	"plot",		NULL,			plot_cmd,		"[equation-number-ranges] [x-range [y-range]] [expressions,]",	"Automatically plot expressions in 2D or 3D with gnuplot.", "Plots variable x; if expression contains y, do a 3D surface plot." },
+{	"plot",		NULL,			plot_cmd,		"[equation-number-ranges] [xyz-ranges] [expressions,]",	"Automatically plot expressions in 2D or 3D with gnuplot.", "Plots variable x; if expression contains y, do a 3D surface plot." },
 #endif
 {	"product",	NULL,			product_cmd,		"variable start end [step-size]",			"Compute the product as variable goes from start to end.", "Related command: sum" },
 #if	READLINE
@@ -101,39 +99,47 @@ static com_type com_list[] = {
 {	"save",		NULL,			save_cmd,		"file-name",						"Save all equation spaces in a text file.", "Related command: read" },
 #endif
 {	"set",		NULL,			set_cmd,		"[[\"no\"] option [value]] ...",			"Display, set, or save current session options.", "\"set\" by itself will show all current option settings." },
-{	"simplify",	NULL,			simplify_cmd,		"[\"sign\" \"symbolic\" \"quick[est]\" \"fraction\"] [equation-number-range]",	"Completely simplify expressions.", "This command may be preceded with \"repeat\"." },
+{	"simplify",	NULL,			simplify_cmd,		"[\"sign\" \"symbolic\" \"quick[est]\" \"fraction\"] [equation-number-ranges]",	"Completely simplify expressions.", "This command may be preceded with \"repeat\"." },
 {	"solve",	NULL,			solve_cmd,		"[\"verify\"] [equation-number-range] [\"for\"] variable or \"0\"",	"Solve the specified equations for a variable or for zero.", "The verify option checks all displayed solutions." },
 {	"sum",		NULL,			sum_cmd,		"variable start end [step-size]",			"Compute the summation as variable goes from start to end.", "Related command: product" },
 #if	!LIBRARY
 {	"tally",	NULL,			tally_cmd,		"[\"average\"]",					"Prompt for and add entries, showing running total and average." },
 #endif
 {	"taylor",	NULL,			taylor_cmd,		"[\"nosimplify\"] variable order point",		"Compute the Taylor series expansion of the current expression." },
-{	"unfactor",	"expand",		unfactor_cmd,		"[\"fraction\" \"quick\" \"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
-{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\"] [equation-number-ranges]",	"Show all variable names used within the specified expressions.", "Related command: code" },
+{	"unfactor",	"expand",		unfactor_cmd,		"[\"count\" \"fraction\" \"quick\" \"power\"] [equation-number-range]",	"Algebraically expand (multiply out) expressions." },
+{	"variables",	NULL,			variables_cmd,		"[\"c\" or \"java\" or \"integer\" or \"count\"] [equation-number-ranges]",	"Show all variable names used within the specified expressions.", "Related command: code" },
 {	"version",	NULL,			version_cmd,		"",							"Display Mathomatic version, compiler, and license information." }
 };
 
 #if	HELP && !LIBRARY
 char	*example_strings[] = {
-	"; The derivative of the absolute value function:\n",
-	"|x|\n",
+	"; Example 1:\n",
+	"; Here the derivative of the absolute value function is computed.\n",
+	"; Expressions are entered by just typing them in:\n",
+	"|x| ; The absolute value of x\n",
 	"derivative ; The result gives the sign of x:\n",
 	"pause\n",
-	"\n",
+	"repeat echo -\n",
+	"; Example 2:\n",
+	"; Here the calculate command is used to plug values into a solved formula.\n",
 	"; A common temperature conversion formula:\n",
 	"fahrenheit = (9*celsius/5) + 32\n",
 	"repeat calculate ; plug in values until an empty line is entered\n",
+	"\n",
 	"; Solve for the other variable and simplify the result:\n",
 	"solve for celsius\n",
 	"simplify\n",
 	"repeat calculate ; plug in values until an empty line is entered\n",
+	"\n",
 	"variables ; show all variables that occur in expressions\n",
 	"pause\n",
-	"\n",
-	"; Expand the following into polynomial form, then refactor:\n",
+	"repeat echo -\n",
+	"; Example 3:\n",
+	"; Expand the following to polynomial form, then refactor and differentiate:\n",
 	"(x+1)^6\n",
-	"unfactor\n",
+	"expand count ; Expand and count the resulting number of terms:\n",
 	"simplify\n",
+	"derivative\n",
 	NULL
 };
 #endif
@@ -412,7 +418,7 @@ do_repeat:
 #define COMPARE_COMMAND_NAME(str)	(len >= min(CMD_REQUIRED_NCHARS, strlen(str)) \
 					&& len <= strlen(str) && strncasecmp(cp, str, len) == 0)
 
-	if (COMPARE_COMMAND_NAME(REPEAT_CMD_STR)) {
+	if (COMPARE_COMMAND_NAME("repeat")) {
 		our_repeat_flag = true;
 		cp = skip_space(cp1);
 		goto do_repeat;
@@ -593,6 +599,30 @@ char	*cp;	/* pointer to variable name string */
 #if	HELP
 #define P(A)	fprintf(gfp, "%s\n", A)
 
+/*
+ * Display command usage info in color.
+ *
+ * Return number of lines displayed.
+ */
+int
+display_usage(pstr, i)
+char	*pstr;	/* prefix string */
+int	i;
+{
+	int	len = 0;
+
+	len += fprintf(gfp, "%s", pstr);
+	set_color(0);
+	len += fprintf(gfp, "%s", com_list[i].name);
+	default_color();
+	len += fprintf(gfp, " %s\n", com_list[i].usage);
+	if (screen_columns && len > screen_columns) {
+		return 2;
+	} else {
+		return 1;
+	}
+}
+
 /*
  * Output command info and usage.
  *
@@ -602,10 +632,10 @@ int
 display_command(i)
 int	i;	/* command table index of command */
 {
-	int	rows = 3;
+	int	rows = 2;
 
 	fprintf(gfp, "%s - %s\n", com_list[i].name, com_list[i].info);
-	fprintf(gfp, "Usage: %s %s\n", com_list[i].name, com_list[i].usage);
+	rows += display_usage("Usage: ", i);
 	if (com_list[i].secondary_name) {
 		fprintf(gfp, "Alternate name for this command: %s\n", com_list[i].secondary_name);
 		rows++;
@@ -615,9 +645,32 @@ int	i;	/* command table index of command */
 		rows++;
 	}
 	fprintf(gfp, "\n");
+#if	DEBUG
+	if (com_list[i].secondary_name && com_list[i].extra) {
+		error_bug("Alternate name and extra info fields both set for this command, only one or the other is currently allowed.");
+	}
+#endif
 	return rows;
 }
 
+/*
+ * Output repeat command info and usage.
+ *
+ * Return the number of lines output.
+ */
+int
+display_repeat_command(void)
+{
+	P("repeat - Automatically repeat the following command over and over.");
+	fprintf(gfp, "Usage: ");
+	set_color(0);
+	fprintf(gfp, "repeat");
+	default_color();
+	fprintf(gfp, " command arguments\n");
+	P("Not all commands are repeatable.\n");
+	return 3;
+}
+
 #if	!LIBRARY
 int
 read_examples(cpp)
@@ -680,6 +733,7 @@ intro:
 		P("Other help keywords: constants, color, license, bugs, geometry.");
 		P("\"help\" or \"?\" followed by a command name will give info on that command.");
 		fprintf(gfp, "These are the %d commands for this version of Mathomatic:\n", ARR_CNT(com_list));
+		set_color(0);
 		for (i = 0; i < ARR_CNT(com_list); i++) {
 			if ((i % 5) == 0)
 				fprintf(gfp, "\n");
@@ -687,6 +741,7 @@ intro:
 			for (; j > 0; j--)
 				fprintf(gfp, " ");
 		}
+		default_color();
 
 		P("\n\nTo select an equation space, type the equation number at the main prompt.");
 		P("To solve the current equation, type the variable name at the main prompt.");
@@ -732,9 +787,7 @@ next_argument:
 			}
 		}
 		if (strncasecmp(cp, "repeat", cp1 - cp) == 0) {
-			P("repeat - Automatically repeat the following command over and over.");
-			P("Usage: repeat command-line");
-			P("Not all commands are repeatable.\n");
+			display_repeat_command();
 			flag = true;
 		}
 		if (flag) {
@@ -778,8 +831,7 @@ next_argument:
 		if (strncasecmp(cp, "usage", cp1 - cp) == 0) {
 			underline_title(P("Mathomatic Command Usage Syntax"));
 			for (i = 0, row = 3; i < ARR_CNT(com_list);) {
-				fprintf(gfp, "%s %s\n", com_list[i].name, com_list[i].usage);
-				row++;
+				row += display_usage("", i);
 				i++;
 				if (i >= ARR_CNT(com_list))
 					break;
@@ -831,7 +883,7 @@ next_argument:
 			P("To enter an expression or equation, simply type it in at the prompt.");
 			P("Operators have precedence decreasing as indicated:\n");
 
-			P("    ! factorial (gamma function)");
+			P("    ! factorial (also gamma() function, highest precedence)");
 			P("    ** or ^ power (exponentiation)");
 			P("    * multiply      / divide        % modulus       // integral divide");
 			P("    + add           - subtract or negate");
@@ -842,7 +894,8 @@ next_argument:
 			P("Valid parentheses characters are () and {}.\n");
 
 			P("Variables consist of any combination of letters, digits, and underscores (_),");
-			P("and \"set special_variable_characters\".  Predefined variables follow:\n");
+			P("and \"set special_variable_characters\".  Variables never start with a digit.");
+			P("Predefined variables follow:\n");
 
 			P("    sign, sign1, sign2, ... - may only be +1 or -1");
 			P("    integer, integer1, ... - may be any integer value\n");
@@ -882,14 +935,19 @@ next_argument:
 			if (color_flag) {
 				fprintf(gfp, "Color mode is currently on, bold mode is currently %s.\n", bold_colors ? "on" : "off");
 				if (html_flag) {
-					fprintf(gfp, "HTML mode is currently on, so color mode is HTML.\n");
+					fprintf(gfp, "HTML mode is currently on, so color mode is HTML.  ");
 				} else {
 #if	WIN32_CONSOLE_COLORS
-					fprintf(gfp, "Using WIN32 CONSOLE color mode.\n");
+					fprintf(gfp, "Using WIN32 CONSOLE color mode.  ");
 #else
-					fprintf(gfp, "Using ANSI color mode.\n");
+					fprintf(gfp, "Using ANSI color mode.  ");
 #endif
 				}
+				if (display_all_colors()) {
+					fprintf(gfp, " are the available colors.\n");
+				} else {
+					fprintf(gfp, "Can't show available colors.\n");
+				}
 			} else {
 				fprintf(gfp, "Color mode is currently turned off.\n");
 			}
