@@ -51,7 +51,7 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
 #endif
 
 #if 	CYGWIN || MINGW
-#undef	UNIX		/* Unix desktop functionality is covered by defining CYGWIN or MINGW */
+#undef	UNIX		/* Unix desktop functionality is slightly different for CYGWIN and MINGW */
 #endif
 
 #if	(UNIX || CYGWIN || MINGW) && !SECURE && !LIBRARY
