@@ -654,10 +654,14 @@ char	*cp;
 	int		trap_flag, singularity, solved;
 	token_type	*ep, *source, *dest;
 	int		n1, *nps, *np;
+	char		*cp_start;
 
+	cp_start = cp;
 	if (current_not_defined()) {
 		return false;
 	}
+	n_tlhs = 0;
+	n_trhs = 0;
 	solved = solved_equation(cur_equation);
 	i = next_espace();
 	if (n_rhs[cur_equation]) {
@@ -687,8 +691,22 @@ char	*cp;
 			error(_("Number of partitions must be a positive, even integer."));
 			return false;
 		}
-		if (extra_characters(cp))
+	}
+	if (*cp) {
+		input_column += (cp - cp_start);
+		cp = parse_expr(tlhs, &n_tlhs, cp, false);
+		if (cp == NULL || n_tlhs <= 0) {
 			return false;
+		}
+	}
+	if (*cp) {
+		cp_start = cp;
+		cp = skip_space(cp);
+		input_column += (cp - cp_start);
+		cp = parse_expr(trhs, &n_trhs, cp, false);
+		if (cp == NULL || extra_characters(cp) || n_trhs <= 0) {
+			return false;
+		}
 	}
 	show_usage = false;
 	if (v == 0) {
@@ -696,6 +714,16 @@ char	*cp;
 			return false;
 		}
 	}
+#if	!SILENT
+	if (debug_level >= 0) {
+		list_var(v, 0);
+		if (n_rhs[cur_equation]) {
+			fprintf(gfp, _("Numerically integrating the RHS wrt (%s)...\n"), var_str);
+		} else {
+			fprintf(gfp, _("Numerically integrating wrt (%s)...\n"), var_str);
+		}
+	}
+#endif
 	singularity = false;
 	for (j = 1; j < *nps; j += 2) {
 		if (source[j].token.operatr == DIVIDE) {
@@ -703,39 +731,43 @@ char	*cp;
 				if (source[k].kind == VARIABLE && source[k].token.variable == v) {
 					singularity = true;
 				}
+				if (source[k].kind == OPERATOR && source[k].level == source[j].level)
+					break;
 			}
 		}
 	}
 	if (singularity) {
 		warning(_("Singularity detected, result of numerical integration might be wrong."));
 	}
-	my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
-	if (!get_expr(tlhs, &n_tlhs)) {
-		return false;
+	if (n_tlhs == 0) {
+		my_strlcpy(prompt_str, _("Enter lower bound: "), sizeof(prompt_str));
+		if (!get_expr(tlhs, &n_tlhs)) {
+			return false;
+		}
 	}
 	subst_constants(tlhs, &n_tlhs);
+	simp_loop(tlhs, &n_tlhs);
 	if (exp_contains_infinity(tlhs, n_tlhs)) {
-		error(_("Error: Bound contains infinity."));
+		error(_("Not computable because: Lower bound contains infinity or NaN."));
 		return false;
 	}
-	my_strlcpy(prompt_str, _("Enter upper bound: "), sizeof(prompt_str));
-	if (!get_expr(trhs, &n_trhs)) {
-		return false;
+	if (n_trhs == 0) {
+		my_strlcpy(prompt_str, _("Enter upper bound: "), sizeof(prompt_str));
+		if (!get_expr(trhs, &n_trhs)) {
+			return false;
+		}
 	}
 	subst_constants(trhs, &n_trhs);
+	simp_loop(trhs, &n_trhs);
 	if (exp_contains_infinity(trhs, n_trhs)) {
-		error(_("Error: Bound contains infinity."));
+		error(_("Not computable because: Upper bound contains infinity or NaN."));
 		return false;
 	}
 	if ((n_tlhs + n_trhs + 3) > n_tokens) {
 		error_huge();
 	}
 #if	!SILENT
-	if (n_rhs[cur_equation]) {
-		fprintf(gfp, _("Approximating the definite integral of the RHS\n"));
-	} else {
-		fprintf(gfp, _("Approximating the definite integral numerically\n"));
-	}
+	fprintf(gfp, _("Approximating the definite integral\n"));
 	if (trap_flag) {
 		fprintf(gfp, _("using the trapezoid method (%d partitions)...\n"), iterations);
 	} else {
@@ -902,11 +934,13 @@ char	*cp;
 	for (; k < n1; k++)
 		dest[k].level++;
 
-	/* simplify and approximate the result as before: */
+	/* simplify and approximate the result even more: */
 	approximate_roots = true;
-	elim_loop(dest, &n1);
-	ufactor(dest, &n1);
-	simp_divide(dest, &n1);
+	do {
+		elim_loop(dest, &n1);
+		ufactor(dest, &n1);
+		simp_divide(dest, &n1);
+	} while (factor_imaginary(dest, &n1));
 	approximate_roots = false;
 
 #if	!SILENT
