@@ -25,7 +25,7 @@
  * and Mathomatic cannot be used.
  */
 int
-matho_init()
+matho_init(void)
 {
 	init_gvars();
 	default_out = stdout;	/* if default_out is a file that is not stdout, output is logged to that file */
