@@ -98,7 +98,7 @@ static char	*html_carray[] = {
  * Subsequent output will have no color until the next call to set_color().
  */
 void
-reset_attr()
+reset_attr(void)
 {
 	fflush(NULL);	/* flush all output */
 	if (color_flag && cur_color >= 0) {
@@ -118,46 +118,52 @@ reset_attr()
 
 /*
  * Set the current color on the display.
- * Subsequent output will have the color in carray[color % ARR_CNT(carray)].
- * Range is 0 to INT_MAX.
+ * Subsequent output will have the color in carray[color % ARR_CNT(carray)] for ANSI color mode.
+ * Other color modes work similarly.
+ * Range for color is 0 to INT_MAX.
+ *
+ * Return actual color number displayed or -1 if no color.
  */
-void
+int
 set_color(color)
 int	color;
 {
+	int	rv = -1;
+
 	if (gfp != stdout)
-		return;
+		return rv;
 	if (cur_color == color)
-		return;
+		return rv;
 	if (color_flag) {
 		if (html_flag) {
 			if (cur_color >= 0) {
 				printf("</font>");
 			}
 			if (bold_colors) {
-				printf("<font color=\"%s\">", bright_html_carray[color % ARR_CNT(bright_html_carray)]);
+				printf("<font color=\"%s\">", bright_html_carray[rv = (color % ARR_CNT(bright_html_carray))]);
 			} else {
-				printf("<font color=\"%s\">", html_carray[color % ARR_CNT(html_carray)]);
+				printf("<font color=\"%s\">", html_carray[rv = (color % ARR_CNT(html_carray))]);
 			}
 		} else {
 #if	WIN32_CONSOLE_COLORS
-                        short attr = windows_carray[color % ARR_CNT(windows_carray)];
+                        short attr = windows_carray[rv = (color % ARR_CNT(windows_carray))];
                         if (bold_colors)
                                  attr |= FOREGROUND_INTENSITY;
                         SetConsoleTextAttribute(hOut, attr);
 #else
-                        printf("\033[%d;%dm", bold_colors, carray[color % ARR_CNT(carray)]);
+                        printf("\033[%d;%dm", bold_colors, carray[rv = (color % ARR_CNT(carray))]);
 #endif
 		}
 		cur_color = color;
 	}
+	return rv;
 }
 
 /*
  * Set normal text color for subsequent output.
  */
 void
-default_color()
+default_color(void)
 {
 	if (gfp == stdout) {
 		if (color_flag && cur_color >= 0) {
@@ -176,6 +182,28 @@ default_color()
 	fflush(stdout);
 }
 
+/*
+ * Display all possible colors for this color mode.
+ *
+ * Return true if successful.
+ */
+int
+display_all_colors(void)
+{
+	int	i, j;
+
+	i = 0;
+	if (set_color(i) < 0)
+		return false;
+	do {
+		printf("#");
+		i++;
+		j = set_color(i);
+	} while (j > 0);
+	default_color();
+	return(j >= 0);
+}
+
 /*
  * Trim the trailing zeros from a string, after the decimal point.
  */
@@ -595,7 +623,11 @@ int		export_flag;	/* flag for exportable format (usually false) */
 			} else if (export_flag || high_prec) {
 				snprintf(buf, sizeof(buf), "%.*g", export_precision, p1[i].token.constant);
 			} else if (finance_option) {
+#if	THOUSANDS_SEPARATOR	/* Fails miserably in MinGW and possibly others, displaying nothing but the format string. */
 				snprintf(buf, sizeof(buf), "%'.*f", finance_option, p1[i].token.constant);
+#else
+				snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+#endif
 			} else {
 				if (p1[i].token.constant < 0.0 && (i + 1) < n && p1[i+1].level == p1[i].level
 				    && (p1[i+1].token.operatr >= POWER)) {
@@ -920,7 +952,6 @@ int			int_flag;	/* integer arithmetic flag, should work with any language */
 static int	cur_line;	/* current line */
 static int	cur_pos;	/* current position in the current line on the screen */
 
-#if	LIBRARY
 /*
  * Return a multi-line C string containing the specified equation space in 2D multi-line fraction format.
  * The string should be freed after use with free(3).
@@ -931,9 +962,10 @@ static int	cur_pos;	/* current position in the current line on the screen */
  * for proper formatting.
  *
  * Return NULL on failure.
- * Result will be limited to TEXT_COLUMNS columns, TEXT_ROWS lines.
- * Exceeding TEXT_COLUMNS will result in truncation,
+ * Result will be limited to current_columns columns, TEXT_ROWS lines.
+ * Exceeding current_columns will result in truncation,
  * exceeding TEXT_ROWS will result in failure and NULL return.
+ * current_columns is set in malloc_vscreen().
  */
 char *
 flist_equation_string(n)
@@ -949,15 +981,17 @@ int	n;	/* equation space number */
 
 	if (n_lhs[n] <= 0)
 		return NULL;
+	if (!malloc_vscreen())
+		return NULL;
 	for (i = 0; i < TEXT_ROWS; i++) {
 		vscreen[i][0] = '\0';
 	}
 	cur_line = 0;
 	cur_pos = 0;
-	len += flist_sub(lhs[n], n_lhs[n], false, NULL, TEXT_COLUMNS, 0, &max_line, &min_line);
+	len = flist_sub(lhs[n], n_lhs[n], false, NULL, current_columns, 0, &max_line, &min_line);
 	if (n_rhs[n]) {
 		len += strlen(EQUATE_STRING);
-		len += flist_sub(rhs[n], n_rhs[n], false, NULL, TEXT_COLUMNS, 0, &high, &low);
+		len += flist_sub(rhs[n], n_rhs[n], false, NULL, current_columns, 0, &high, &low);
 		if (high > max_line)
 			max_line = high;
 		if (low < min_line)
@@ -967,16 +1001,16 @@ int	n;	/* equation space number */
 		return NULL;
 	for (cur_line = max_line, screen_line = 0; cur_line >= min_line; cur_line--, screen_line++) {
 		pos = cur_pos = 0;
-		pos += flist_sub(lhs[n], n_lhs[n], true, vscreen[screen_line], TEXT_COLUMNS, pos, &high, &low);
+		pos += flist_sub(lhs[n], n_lhs[n], true, vscreen[screen_line], current_columns, pos, &high, &low);
 		if (n_rhs[n]) {
 			if (cur_line == 0) {
 				cur_pos += strlen(EQUATE_STRING);
 				cur_len = strlen(vscreen[screen_line]);
-				if ((TEXT_COLUMNS - cur_len) > 0)
-					my_strlcpy(&vscreen[screen_line][cur_len], EQUATE_STRING, TEXT_COLUMNS - cur_len);
+				if ((current_columns - cur_len) > 0)
+					my_strlcpy(&vscreen[screen_line][cur_len], EQUATE_STRING, current_columns - cur_len);
 			}
 			pos += strlen(EQUATE_STRING);
-			pos += flist_sub(rhs[n], n_rhs[n], true, vscreen[screen_line], TEXT_COLUMNS, pos, &high, &low);
+			pos += flist_sub(rhs[n], n_rhs[n], true, vscreen[screen_line], current_columns, pos, &high, &low);
 		}
 	}
 	if (screen_line <= 0)
@@ -997,7 +1031,6 @@ int	n;	/* equation space number */
 	}
 	return(cp);
 }
-#endif
 
 /*
  * Display an equation space in 2D multi-line fraction format.
@@ -1020,9 +1053,15 @@ int	n;	/* equation space number */
 	int	high = 0, low = 0;
 	int	max_line = 0, min_line = 0;
 	int	max2_line = 0, min2_line = 0;
+	int	use_screen_columns;
 
 	if (n_lhs[n] <= 0)
 		return 0;
+#if	ROBOT_COMMAND
+	use_screen_columns = true;
+#else
+	use_screen_columns = (gfp == stdout);
+#endif
 	len = snprintf(buf, sizeof(buf), "#%d: ", n + 1);
 	cur_line = 0;
 	cur_pos = 0;
@@ -1032,7 +1071,7 @@ int	n;	/* equation space number */
 		len += strlen(EQUATE_STRING);
 make_smaller:
 		len2 = flist_sub(rhs[n], sind, false, NULL, screen_columns, 0, &high, &low);
-		if (screen_columns && gfp == stdout && (len + len2) >= screen_columns && sind > 0) {
+		if (screen_columns && use_screen_columns && (len + len2) >= screen_columns && sind > 0) {
 			for (sind--; sind > 0; sind--) {
 				if (rhs[n][sind].level == 1 && rhs[n][sind].kind == OPERATOR) {
 					switch (rhs[n][sind].token.operatr) {
@@ -1055,7 +1094,7 @@ make_smaller:
 		len3 = 0;
 	}
 	width = max(len + len2, len3);
-	if (screen_columns && gfp == stdout && width >= screen_columns) {
+	if (screen_columns && use_screen_columns && width >= screen_columns) {
 		/* output too wide to fit screen, output in single-line format */
 		width = list1_sub(n, false);
 #if	CYGWIN || MINGW
@@ -1308,9 +1347,17 @@ check_again:
 				len += snprintf(buf, sizeof(buf), "-");
 			} else if (finance_option) {
 				if (p1[i].token.constant < 0.0) {
+#if	THOUSANDS_SEPARATOR
 					len += snprintf(buf, sizeof(buf), "(%'.*f)", finance_option, p1[i].token.constant);
+#else
+					len += snprintf(buf, sizeof(buf), "(%.*f)", finance_option, p1[i].token.constant);
+#endif
 				} else {
+#if	THOUSANDS_SEPARATOR
 					len += snprintf(buf, sizeof(buf), "%'.*f", finance_option, p1[i].token.constant);
+#else
+					len += snprintf(buf, sizeof(buf), "%.*f", finance_option, p1[i].token.constant);
+#endif
 				}
 			} else {
 				if (p1[i].token.constant < 0.0 && (i + 1) < n && p1[i+1].level == p1[i].level
