@@ -376,7 +376,7 @@ char	**argv;
  * Repeatedly read a line of text from standard input and process the expression or command.
  */
 void
-main_io_loop()
+main_io_loop(void)
 {
 	char	*cp = NULL;
 
@@ -395,7 +395,7 @@ main_io_loop()
  * Return true on success.
  */
 int
-set_signals()
+set_signals(void)
 {
 	int	rv = true;
 
@@ -452,7 +452,7 @@ char	*cp;	/* string containing filename to modify */
  * otherwise return true.
  */
 int
-load_rc()
+load_rc(void)
 {
 	FILE	*fp = NULL;
 	char	buf[MAX_CMD_LEN];
