@@ -51,7 +51,7 @@ put_up_arrow(cnt, cp)
 int	cnt;	/* position of error, relative to "input_column" */
 char	*cp;	/* error message (constant ASCII string) */
 {
-#if	!SILENT
+#if	!SILENT && !LIBRARY
 	int	 i;
 
 	cnt += input_column;
@@ -492,11 +492,11 @@ parse_power:
 			switch (*cp) {
 			case '+':
 			case '-':
-				i = decstrtol(cp, &cp1);
+				i = strtol(cp, &cp1, 10);
 				i = cur_equation + i;
 				break;
 			default:
-				i = decstrtol(cp, &cp1) - 1;
+				i = strtol(cp, &cp1, 10) - 1;
 				break;
 			}
 			if (cp == cp1) {
