@@ -527,7 +527,7 @@ int		*np, loc1, len, level;
  * If this returns false, nothing is removed.
  */
 int
-remove_factors()
+remove_factors(void)
 {
 	int	i, j, k;
 	int	plus_flag = false, divide_flag = false;
@@ -1346,7 +1346,7 @@ next_thingy:
  * negative if result is larger.
  * Quotient is returned in tlhs[] and remainder in trhs[].
  *
- * If *vp is 0, automatically select the best base variable and return it in *vp.
+ * If *vp is 0, automatically select the best polynomial base variable and return it in *vp.
  */
 int
 poly_div(d1, len1, d2, len2, vp)
@@ -1354,7 +1354,7 @@ token_type	*d1;		/* pointer to dividend */
 int		len1;		/* length of dividend */
 token_type	*d2;		/* pointer to divisor */
 int		len2;		/* length of divisor */
-long		*vp;		/* pointer to base variable */
+long		*vp;		/* pointer to polynomial base variable */
 {
 	int		i;
 	int		rv;
@@ -1420,7 +1420,7 @@ long		*vp;
 	uf_simp(trhs, &n_trhs);
 	uf_simp(tlhs, &n_tlhs);
 	if (*vp == 0) {
-		/* Select the best base variable. */
+		/* Select the best polynomial base variable. */
 		if (!find_highest_count(trhs, n_trhs, tlhs, n_tlhs, vp))
 			return false;
 	}
@@ -1878,7 +1878,9 @@ int		*tp1, *lentp1;
 /*
  * This routine automatically finds the best normal variable to do polynomial division with.
  *
- * Returns 0 if nothing was found, otherwise the selected base variable is returned in *vp1.
+ * Returns 0 if nothing was found,
+ * otherwise the selected polynomial base variable is returned in *vp1,
+ * and the number of times it occurs in the dividend is returned.
  */
 static int
 find_highest_count(p1, n1, p2, n2, vp1)
@@ -1886,7 +1888,7 @@ token_type	*p1;		/* pointer to dividend expression */
 int		n1;		/* length of dividend */
 token_type	*p2;		/* pointer to divisor expression */
 int		n2;		/* length of divisor */
-long		*vp1;		/* variable pointer */
+long		*vp1;		/* variable pointer to return base variable with */
 {
 	int		i;
 	int		vc, cnt;
@@ -2020,13 +2022,13 @@ int		n1, loc;
  * with ordering and much flexibility.
  *
  * The number of terms raised to the highest power is returned, unless *vp1 == 0,
- * then 0 is returned and *vp1 is set to a valid base variable, if any.
+ * then 0 is returned and *vp1 is set to a valid polynomial base variable, if any.
  */
 int
 find_greatest_power(p1, n1, vp1, pp1, tp1, lentp1, dcodep)
 token_type	*p1;		/* pointer to expression */
 int		n1;		/* length of expression */
-long		*vp1;		/* base variable */
+long		*vp1;		/* polynomial base variable */
 double		*pp1;		/* returned power of the returned term, which will be the highest power */
 int		*tp1, *lentp1;	/* the returned term index and length */
 int		*dcodep;	/* divide flag pointer indicates if term is a denominator; */
