@@ -56,7 +56,7 @@ void	allocate_triangle(void);
 void	calculate_triangle(void);
 void	display_triangle(void);
 int	center_buf(int line_no, int cell_size);
-void	usage(void);
+void	usage(int ev);
 
 int		glines = 26;
 int		gcell_size = 6;
@@ -95,7 +95,7 @@ main(int argc, char *argv[])
 			break;
 		}
 	default:
-		usage();
+		usage(EXIT_FAILURE);
 	}
 	if (glines <= 0 || glines > MAX_LINES) {
 		fprintf(stderr, "%s: Number of lines out of range (1..%d).\n", prog_name, MAX_LINES);
@@ -211,9 +211,10 @@ center_buf(int line_no, int cell_size)
 }
 
 void
-usage(void)
+usage(int ev)
 {
 	printf("Usage: %s [number-of-lines]\n\n", prog_name);
 	printf("Display up to %d lines of Pascal's triangle.\n", MAX_LINES);
-	exit(EXIT_FAILURE);
+	printf("If number-of-lines is specified, don't center output.\n");
+	exit(ev);
 }
