@@ -34,8 +34,9 @@ George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  * 10/25/10 - Added -c and -h options.
  * 10/26/10 - Using usage2() instead of usage() most of the time, now.
  * 10/28/10 - Fixed to work with 16-bit integers.
- * 1/16/11 - Allow to use doubles instead of long doubles with USE_DOUBLES define, for systems that don't completely support long doubles.
- * 5/10/11 - Added -u (unbuffered output) option.
+ * 1/16/11 - Allow to use doubles instead of long doubles with USE_DOUBLES define, for systems that don't fully support long doubles.
+ * 5/10/11 - Added -u (unbuffered output) option, for real-time output.
+ * 11/13/11 - Use EXIT_SUCCESS and EXIT_FAILURE macros.
  */
 
 #include <stdio.h>
@@ -160,11 +161,11 @@ do_again:
 		case 'c':
 			count_requested = true;
 			if (optarg && !get_double_type_int(optarg, &number)) {
-				usage2(2);
+				usage2(EXIT_FAILURE);
 			}
 			break;
 		case 'h':
-			usage2(0);
+			usage2(EXIT_SUCCESS);
 			break;
 		case 't':
 			twin_flag = true;
@@ -172,7 +173,7 @@ do_again:
 		case 'p':
 			pal_flag = true;
 			if (optarg && !get_double_type_int(optarg, &pal_base)) {
-				usage2(2);
+				usage2(EXIT_FAILURE);
 			}
 			break;
 		case 'u':
@@ -180,7 +181,7 @@ do_again:
 			setbuf(stderr, NULL);
 			break;
 		default:
-			usage2(2);
+			usage2(EXIT_FAILURE);
 		}
 	}
 	if (argc > optind) {
@@ -193,19 +194,19 @@ do_again:
 		if (get_double_type_int(argv[optind], &start_value)) {
 			optind++;
 		} else {
-			usage2(2);
+			usage2(EXIT_FAILURE);
 		}
 		if (argc > optind && isdigit(argv[optind][0])) {
 			if (get_double_type_int(argv[optind], &end_value)) {
 				if (end_value < start_value) {
 					fprintf(stderr, "End value is less than start value.\n");
-					usage2(2);
+					usage2(EXIT_FAILURE);
 				}
 				optind++;
 				if (number == 0)
 					number = max_integer;
 			} else {
-				usage2(2);
+				usage2(EXIT_FAILURE);
 			}
 		}
 	}
@@ -221,11 +222,11 @@ do_again:
 			optind++;
 		} else {
 			fprintf(stderr, "Unrecognized argument: \"%s\".\n", argv[optind]);
-			usage(2);
+			usage(EXIT_FAILURE);
 		}
 		if (argc > optind && isdigit(argv[optind][0])) {
 			if (!get_double_type_int(argv[optind], &pal_base)) {
-				usage(2);
+				usage(EXIT_FAILURE);
 			}
 			optind++;
 		}
@@ -238,17 +239,17 @@ do_again:
 	}
 	if (argc > optind) {
 		fprintf(stderr, "Unrecognized argument: \"%s\".\n", argv[optind]);
-		usage(2);
+		usage(EXIT_FAILURE);
 	}
 	if (pal_base < 2 || pal_base >= INT_MAX) {
 		fprintf(stderr, "Palindrome number base must be >= 2.\n");
-		usage(2);
+		usage(EXIT_FAILURE);
 	}
 	if (start_value < 0.0) {
 get1:
 		fprintf(stderr, "Enter number to start finding consecutive primes at (0): ");
 		if (fgets(buf, sizeof(buf), stdin) == NULL)
-			exit(1);
+			exit(EXIT_SUCCESS);
 		switch (buf[0]) {
 		case '\0':
 		case '\n':
@@ -263,7 +264,7 @@ get1:
 	}
 	if (number == 0) {
 get2:
-		fprintf(stderr, "Enter number of%s%s primes to output",
+		fprintf(stderr, "Enter number of%s%s primes to output (0 to end)",
 		    pal_flag ? " palindromic" : "", twin_flag ? " twin" : "");
 #if	USE_DOUBLES
 		fprintf(stderr, " (%g): ", default_number);
@@ -271,7 +272,7 @@ get2:
 		fprintf(stderr, " (%Lg): ", default_number);
 #endif
 		if (fgets(buf, sizeof(buf), stdin) == NULL)
-			exit(1);
+			exit(EXIT_SUCCESS);
 		switch (buf[0]) {
 		case '\0':
 		case '\n':
@@ -288,14 +289,14 @@ get2:
 	if (prime == NULL) {
 /* allocate the prime[] buffer: */
 		buffer_size = BUFFER_SIZE;
-#if	DEBUG
-		fprintf(stderr, "buffer_size = %d\n", buffer_size);
+#if	0
+		fprintf(stderr, "buffer_size = %d bytes.\n", buffer_size);
 #endif
 		prime = (char *) malloc(buffer_size);
 	}
 	if (prime == NULL) {
 		fprintf(stderr, "%s: Not enough memory for buffer_size = %d.\n", prog_name, buffer_size);
-		exit(2);
+		exit(EXIT_FAILURE);
 	}
 	generate_primes();
 #if	MINGW || __APPLE__
@@ -304,7 +305,7 @@ get2:
 		goto do_again;
 	}
 #endif
-	exit(0);
+	exit(EXIT_SUCCESS);
 }
 
 /*
@@ -416,7 +417,7 @@ generate_primes(void)
 check_return:
 	if (count_requested && count < number) {
 		fprintf(stderr, "%s: Number of primes requested not reached.\n", prog_name);
-		exit(1);
+		exit(EXIT_FAILURE);
 	}
 }
 
