@@ -167,13 +167,21 @@ main(int argc, char *argv[])
 				perror(NULL);
 				exit(EXIT_FAILURE);
 			}
-			if (argv[i][0] == '\0' || *cp || d1 <= 0) {
+			if (d1 <= 0) {
+				fprintf(stderr, "Invalid command-line argument: \"%s\", positive integer required.\n", argv[i]);
+				exit(EXIT_FAILURE);
+			}
+			if (*cp == '+') {
+				goto count_up;
+			}
+			if (argv[i][0] == '\0' || *cp) {
 				fprintf(stderr, "Invalid number: \"%s\".\n", argv[i]);
 				exit(EXIT_FAILURE);
 			}
 			findsq(d1);
 		}
 	} else {
+count_up:
 		for (;; d1 += 1) {
 			findsq(d1);
 		}
