@@ -9,6 +9,7 @@ void error_huge(void);
 void error_bug(const char *str);
 void check_err(void);
 int get_screen_size(void);
+int malloc_vscreen(void);
 int init_mem(void);
 void init_gvars(void);
 void clean_up(void);
@@ -155,7 +156,9 @@ int process_rv(char *cp);
 int display_process(char *cp);
 int shell_out(char *cp);
 char *parse_var2(long *vp, char *cp);
+int display_usage(char *pstr, int i);
 int display_command(int i);
+int display_repeat_command(void);
 int read_examples(char **cpp);
 void underline_title(int count);
 int help_cmd(char *cp);
@@ -167,8 +170,9 @@ int laplace_cmd(char *cp);
 int nintegrate_cmd(char *cp);
 /* list.c */
 void reset_attr(void);
-void set_color(int color);
+int set_color(int color);
 void default_color(void);
+int display_all_colors(void);
 int list1_sub(int n, int export_flag);
 int list_sub(int n);
 void list_debug(int level, token_type *p1, int n1, token_type *p2, int n2);
@@ -183,6 +187,7 @@ int int_expr(token_type *p1, int n);
 int list_code_equation(int en, enum language_list language, int int_flag);
 char *string_code_equation(int n, enum language_list language, int int_flag);
 int list_code(token_type *equation, int *np, int outflag, char *string, enum language_list language, int int_flag);
+char *flist_equation_string(int n);
 int flist_equation(int n);
 /* main.c */
 void display_startup_message(FILE *fp);
@@ -254,6 +259,7 @@ void simpb_side(token_type *equation, int *np, int uf_power_flag, int power_flag
 void simple_frac_side(token_type *equation, int *np);
 void simpa_side(token_type *equation, int *np, int quick_flag, int frac_flag);
 void simpa_repeat_side(token_type *equation, int *np, int quick_flag, int frac_flag);
+void simpa_repeat(int n, int quick_flag, int frac_flag);
 void simple_frac_repeat_side(token_type *equation, int *np);
 int simp_loop(token_type *equation, int *np);
 int simp_pp(token_type *equation, int *np);
