@@ -49,6 +49,11 @@ organize(equation, np)
 token_type	*equation;	/* equation side pointer */
 int		*np;		/* pointer to length of equation side */
 {
+#if	DEBUG
+	if (equation == NULL || np == NULL) {
+		error_bug("NULL pointer passed to organize().");
+	}
+#endif
 	if (*np <= 0 || (*np & 1) == 0) {
 		printf("Bad expression size = %d.\n", *np);
 		error_bug("Internal error: organize() called with bad expression size.");
@@ -807,6 +812,25 @@ int		frac_flag;	/* "simplify fraction" option, simplify to the ratio of two poly
 	}
 }
 
+/*
+ * Completely and repeatedly (if repeat_flag) simplify an equation space to the smallest possible size.
+ *
+ * Globals tes[], tlhs[], and trhs[] are clobbered.
+ */
+void
+simpa_repeat(n, quick_flag, frac_flag)
+int	n;		/* equation space number to simplify */
+int	quick_flag;	/* "simplify quick" option, simpler fractions with no (x+1)^2 expansion */
+int	frac_flag;	/* "simplify fraction" option, simplify to the ratio of two polynomials */
+{
+	if (n_lhs[n] <= 0)
+		return;
+	simpa_repeat_side(lhs[n], &n_lhs[n], quick_flag, frac_flag);
+	if (n_rhs[n] > 0) {
+		simpa_repeat_side(rhs[n], &n_rhs[n], quick_flag, frac_flag);
+	}
+}
+
 /*
  * This routine is used by the fraction command.
  * It repeats the simplification until the smallest expression is achieved,
@@ -1780,6 +1804,11 @@ int		*diff_signp;	/* different sign flag pointer */
 {
 	int	l1, l2;
 
+#if	DEBUG
+	if (n1 < 1 || n2 < 1 || (n1 & 1) != 1 || (n2 & 1) != 1 || diff_signp == NULL || p1 == NULL || p2 == NULL) {
+		error_bug("Programming error in call to se_compare().");
+	}
+#endif
 #if	1
 	if (((n1 > n2) ? ((n1 + 1) / (n2 + 1)) : ((n2 + 1) / (n1 + 1))) > 3) {
 		/* expressions are grossly different in size, no need to compare, they are different */
