@@ -636,7 +636,7 @@ long	v;
 
 #if	!SILENT
 	if (debug_level >= 0) {
-		fprintf(gfp, _("Raising both equation sides to the power of %.*g and unfactoring...\n"), precision, 1.0 / d);
+		fprintf(gfp, _("Raising both equation sides to the power of %.*g and expanding...\n"), precision, 1.0 / d);
 	}
 #endif
 	list_tdebug(2);
