@@ -1,28 +1,31 @@
 /*
  * Group and combine algebraic fractions for Mathomatic.
  *
- * Copyright (C) 1987-2010 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
  
-    This library is free software; you can redistribute it and/or
-    modify it under the terms of the GNU Lesser General Public 
-    License as published by the Free Software Foundation; either 
-    version 2.1 of the License, or (at your option) any later version.
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
 
-    This library is distributed in the hope that it will be useful,
-    but WITHOUT ANY WARRANTY; without even the implied warranty of 
-    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
-    Lesser General Public License for more details.
- 
-    You should have received a copy of the GNU Lesser General Public 
-    License along with this library; if not, write to the Free Software 
-    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
  
  */
 
 #include "includes.h"
 
-static int	sf_recurse();
-static int	sf_sub();
+static int sf_recurse(token_type *equation, int *np, int loc, int level, int start_flag);
+static int sf_sub(token_type *equation, int *np, int loc, int i1, int n1, int i2, int n2, int level, int check_flag);
 
 static void
 group_recurse(equation, np, loc, level)
@@ -33,7 +36,7 @@ int		level;		/* current level of parentheses within the sub-expression starting
 {
 	int		i;
 	int		len;
-	int		di, edi;
+	int		di = -1, edi;
 	int		group_flag = false;
 	int		e1;
 
@@ -47,9 +50,7 @@ int		level;		/* current level of parentheses within the sub-expression starting
 		}
 		i++;
 	}
-	e1 = i;
-	di = -1;
-	edi = e1;
+	edi = e1 = i;
 	for (i = loc + 1; i < e1; i += 2) {
 		if (equation[i].level == level) {
 			if (equation[i].token.operatr == DIVIDE) {
@@ -82,7 +83,7 @@ int		level;		/* current level of parentheses within the sub-expression starting
 	if (group_flag) {
 		for (i = di + 1; i < edi; i++) {
 			if (equation[i].level == level && equation[i].kind == OPERATOR) {
-#if	0
+#if	DEBUG
 				if (equation[i].token.operatr != DIVIDE) {
 					error_bug("Bug in group_recurse().");
 				}
@@ -237,7 +238,18 @@ sf_again:
 		for (k = j + 1; k < *np && equation[k].level > level; k += 2)
 			;
 		len2 = k - j;
+#if	0
+		side_debug(0, &equation[i], len1);
+		side_debug(0, &equation[j], len2);
+#endif
 		if (sf_sub(equation, np, loc, i, len1, j, len2, level + 1, start_flag == 2)) {
+#if	0
+			int junk;
+			debug_string(0, "super_factor() successful:");
+			for (junk = 1; (loc + junk) < *np && equation[loc+junk].level > level; junk += 2)
+				;
+			side_debug(0, &equation[loc], junk);
+#endif
 			modified = true;
 			goto sf_again;
 		}
@@ -258,6 +270,7 @@ int		*np, loc, i1, n1, i2, n2, level, check_flag;
 	token_type	*p1, *p2;
 	int		np1, np2;
 	int		div_flag1 = false, div_flag2 = false;
+	int		rv;
 
 	e1 = i1 + n1;
 	e2 = i2 + n2;
@@ -296,20 +309,28 @@ int		*np, loc, i1, n1, i2, n2, level, check_flag;
 	if (!div_flag1 && !div_flag2)
 		return false;
 	if (check_flag && div_flag1 && div_flag2) {
-		if (poly2_gcd(&equation[b1], i - b1, &equation[b2], j - b2, 0L)) {
+#if	DEBUG
+		debug_string(1, "Trying to find a polynomial GCD between 2 denominators in sf_sub():");
+		side_debug(1, &equation[b1], i - b1);
+		side_debug(1, &equation[b2], j - b2);
+#endif
+		if ((rv = poly2_gcd(&equation[b1], i - b1, &equation[b2], j - b2, 0L, true)) > 0) {
 			p1 = tlhs;
 			np1 = n_tlhs;
 			p2 = trhs;
 			np2 = n_trhs;
-			goto do_gcd;
+			goto do_gcd_super;
 		}
-		if (poly2_gcd(&equation[b2], j - b2, &equation[b1], i - b1, 0L)) {
+		if (rv == 0 && poly2_gcd(&equation[b2], j - b2, &equation[b1], i - b1, 0L, true) > 0) {
 			p1 = trhs;
 			np1 = n_trhs;
 			p2 = tlhs;
 			np2 = n_tlhs;
-			goto do_gcd;
+			goto do_gcd_super;
 		}
+#if	DEBUG
+		debug_string(1, "Done; polynomial GCD not found.");
+#endif
 	}
         if (n1 + n2 + (i - b1) + (j - b2) + 8 > n_tokens) {
                 error_huge();
@@ -322,7 +343,7 @@ int		*np, loc, i1, n1, i2, n2, level, check_flag;
 		for (k = i2; k < e2; k++)
 			equation[k].level++;
 	}
-	len = b1 - i1 - 1;
+	len = (b1 - i1) - 1;
 	blt(scratch, &equation[i1], len * sizeof(token_type));
 	if (op1 == MINUS) {
 		scratch[len].level = level;
@@ -404,7 +425,11 @@ end_mess:
 	blt(&equation[i1], scratch, len * sizeof(token_type));
 	return true;
 
-do_gcd:
+do_gcd_super:
+#if	DEBUG
+	debug_string(1, "Found a polynomial GCD between 2 denominators in sf_sub().");
+#endif
+
 	if (5 - i1 + e1 + (2*np2) + b2 - i2 + e2 - j + np1 > n_tokens) {
 		error_huge();
 	}
@@ -414,7 +439,7 @@ do_gcd:
 	for (k = 0; k < np2; k++) {
 		p2[k].level += level;
 	}
-	len = b1 - i1 - 1;
+	len = (b1 - i1) - 1;
 	blt(scratch, &equation[i1], len * sizeof(token_type));
 	if (op1 == MINUS) {
 		scratch[len].level = level;
