@@ -1,17 +1,35 @@
 /*
  * Mathomatic unfactorizing (expanding) routines.
  *
- * Copyright (C) 1987-2009 George Gesslein II.
+ * Copyright (C) 1987-2011 George Gesslein II.
+ 
+This library is free software; you can redistribute it and/or
+modify it under the terms of the GNU Lesser General Public
+License as published by the Free Software Foundation; either
+version 2.1 of the License, or (at your option) any later version.
+
+This library is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+Lesser General Public License for more details.
+
+You should have received a copy of the GNU Lesser General Public
+License along with this library; if not, write to the Free Software
+Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
+
+The chief copyright holder can be contacted at gesslein@linux.com, or
+George Gesslein II, P.O. Box 224, Lansing, NY  14882-0224  USA.
+ 
  */
 
 #include "includes.h"
 
-static int	unf_sub();
+static int unf_sub(token_type *equation, int *np, int b1, int loc, int e1, int level, int ii);
 
 /*
  * Unfactor times and divide only (products of sums) and simplify.
  *
- * Return true if equation side was modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_tsimp(equation, np)
@@ -33,42 +51,42 @@ int		*np;
  * Unfactor power only.
  * (a * b)^c -> a^c * b^c
  *
- * Return true if equation side is modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_power(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	rv;
+	int	count = -1;
 
-	organize(equation, np);
-	rv = sub_ufactor(equation, np, 2);
-	if (rv) {
+	do {
 		organize(equation, np);
-	}
-	return rv;
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 2));
+	return count;
 }
 
 /*
  * Unfactor power only.
  * a^(b + c) -> a^b * a^c
  *
- * Return true if equation side is modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_pplus(equation, np)
 token_type	*equation;
 int		*np;
 {
-	int	rv;
+	int	count = -1;
 
-	organize(equation, np);
-	rv = sub_ufactor(equation, np, 4);
-	if (rv) {
+	do {
 		organize(equation, np);
-	}
-	return rv;
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 4));
+	return count;
 }
 
 /*
@@ -98,10 +116,13 @@ uf_repeat(equation, np)
 token_type	*equation;
 int		*np;
 {
-	organize(equation, np);
-	if (sub_ufactor(equation, np, 6)) {
+	int	count = -1;
+
+	do {
 		organize(equation, np);
-	}
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 6));
 	patch_root_div(equation, np);
 }
 
@@ -115,10 +136,13 @@ uf_repeat_always(equation, np)
 token_type	*equation;
 int		*np;
 {
-	organize(equation, np);
-	if (sub_ufactor(equation, np, 8)) {
+	int	count = -1;
+
+	do {
 		organize(equation, np);
-	}
+		if (++count > 0)
+			break;
+	} while (sub_ufactor(equation, np, 8));
 }
 
 /*
@@ -164,6 +188,9 @@ int		*np;
 	return rv;
 }
 
+/*
+ * Increase the level of numerators by 2, so that the divide operator is not unfactored.
+ */
 static void
 no_divide(equation, np)
 token_type	*equation;
@@ -189,7 +216,7 @@ int		*np;
  * (a + b)*c -> a*c + b*c
  * If partial_flag is true, don't expand (a+b)/(c+d) nor (a+b)/c.
  *
- * Return true if equation side was modified.
+ * Return true if equation side was unfactored.
  */
 int
 uf_times(equation, np)
@@ -201,8 +228,8 @@ int		*np;
 
 	do {
 		organize(equation, np);
-		if (partial_flag) {
-			reorder(equation, np);
+		if (reorder(equation, np)) {
+			organize(equation, np);
 		}
 		group_proc(equation, np);
 		if (partial_flag) {
@@ -453,10 +480,6 @@ do_repeat:
 				break;
 			if ((i - b1) == 1 && equation[b1].kind != CONSTANT)
 				break;
-#if	false
-			if ((i - b1) > 1 && level1_plus_count(&equation[b1], (i - b1)) <= 0)
-				break;
-#endif
 			if ((i - b1) > 1 && d1 > 2.0 && fmod(d1, 1.0) != 0.0)
 				break;
 		}
@@ -556,9 +579,11 @@ int		*np;
 	return modified;
 }
 
-#if	false	/* the following is not currently used */
+#if	0	/* the following is not currently used */
 /*
  * Convert a/(x^y) to a*(1/x)^y.
+ * If y is a constant, don't do.
+ *
  * Return true if equation side is modified.
  */
 int
@@ -623,6 +648,8 @@ int		*np, i;
 /*
  * Convert anything times a negative constant to a positive constant
  * divided by -1.
+ * When uf_times() is done after this, the additive denominators will be
+ * attempted to be negated, possibly getting rid of unneeded times -1.
  */
 void
 uf_neg_help(equation, np)
@@ -663,7 +690,7 @@ int		*np;
 /*
  * Simplify division by irrational constants (roots like 2^.5) by
  * converting 1/(k1^k2) to 1/(k1^(k2-1))/k1 if k1 is integer,
- * otherwise 1*((1/k1)^k2).
+ * otherwise convert 1/(k1^k2) to 1*((1/k1)^k2).
  *
  * Return true if equation side was modified.
  */
@@ -685,7 +712,8 @@ int		*np;
 			    && equation[i+3].level == level
 			    && equation[i+3].kind == CONSTANT) {
 				if (fmod(equation[i+1].token.constant, 1.0) == 0.0) {
-					if (!isfinite(equation[i+3].token.constant)
+					if (!rationalize_denominators
+					    || !isfinite(equation[i+3].token.constant)
 					    || equation[i+3].token.constant <= 0.0
 					    || equation[i+3].token.constant >= 1.0) {
 						continue;
